@@ -6,12 +6,12 @@
 @author  Michael Behrisch
 @author  Jakob Erdmann
 @date    2009-03-26
-@version $Id: runner.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: runner.py 18096 2015-03-17 09:50:59Z behrisch $
 
 Tutorial for traffic light control via the TraCI interface.
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2009-2014 DLR/TS, Germany
+Copyright (C) 2009-2015 DLR/TS, Germany
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -20,37 +20,43 @@ the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
 
-import os, sys
+import os
+import sys
 import optparse
 import subprocess
 import random
 
 # we need to import python modules from the $SUMO_HOME/tools directory
 try:
-    sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..', "tools")) # tutorial in tests
-    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(os.path.dirname(__file__), "..", "..", "..")), "tools")) # tutorial in docs
+    sys.path.append(os.path.join(os.path.dirname(
+        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
+    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
+        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
     from sumolib import checkBinary
 except ImportError:
-    sys.exit("please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")
+    sys.exit(
+        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")
 
 import traci
 # the port used for communicating with your sumo instance
 PORT = 8873
 
-NSGREEN = "GrGr" 
+NSGREEN = "GrGr"
 NSYELLOW = "yryr"
-WEGREEN = "rGrG" 
+WEGREEN = "rGrG"
 WEYELLOW = "ryry"
 
-PROGRAM = [WEYELLOW,WEYELLOW,WEYELLOW,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSYELLOW,NSYELLOW,WEGREEN]
+PROGRAM = [WEYELLOW, WEYELLOW, WEYELLOW, NSGREEN, NSGREEN, NSGREEN,
+           NSGREEN, NSGREEN, NSGREEN, NSGREEN, NSGREEN, NSYELLOW, NSYELLOW, WEGREEN]
+
 
 def generate_routefile():
-    random.seed(42) # make tests reproducible
-    N = 3600 # number of time steps
+    random.seed(42)  # make tests reproducible
+    N = 3600  # number of time steps
     # demand per second from different directions
-    pWE = 1./10 
-    pEW = 1./11
-    pNS = 1./30
+    pWE = 1. / 10
+    pEW = 1. / 11
+    pNS = 1. / 30
     with open("data/cross.rou.xml", "w") as routes:
         print >> routes, """<routes>
         <vType id="typeWE" accel="0.8" decel="4.5" sigma="0.5" length="5" minGap="2.5" maxSpeed="16.67" guiShape="passenger"/>
@@ -62,32 +68,37 @@ def generate_routefile():
         lastVeh = 0
         vehNr = 0
         for i in range(N):
-            if random.uniform(0,1) < pWE:
-                print >> routes, '    <vehicle id="right_%i" type="typeWE" route="right" depart="%i" />' % (vehNr, i)
+            if random.uniform(0, 1) < pWE:
+                print >> routes, '    <vehicle id="right_%i" type="typeWE" route="right" depart="%i" />' % (
+                    vehNr, i)
                 vehNr += 1
                 lastVeh = i
-            if random.uniform(0,1) < pEW:
-                print >> routes, '    <vehicle id="left_%i" type="typeWE" route="left" depart="%i" />' % (vehNr, i)
+            if random.uniform(0, 1) < pEW:
+                print >> routes, '    <vehicle id="left_%i" type="typeWE" route="left" depart="%i" />' % (
+                    vehNr, i)
                 vehNr += 1
                 lastVeh = i
-            if random.uniform(0,1) < pNS:
-                print >> routes, '    <vehicle id="down_%i" type="typeNS" route="down" depart="%i" color="1,0,0"/>' % (vehNr, i)
+            if random.uniform(0, 1) < pNS:
+                print >> routes, '    <vehicle id="down_%i" type="typeNS" route="down" depart="%i" color="1,0,0"/>' % (
+                    vehNr, i)
                 vehNr += 1
                 lastVeh = i
         print >> routes, "</routes>"
 
+
 def run():
     """execute the TraCI control loop"""
     traci.init(PORT)
-    programPointer = len(PROGRAM)-1
+    programPointer = len(PROGRAM) - 1
     step = 0
     while traci.simulation.getMinExpectedNumber() > 0:
         traci.simulationStep()
-        programPointer = min(programPointer+1, len(PROGRAM)-1)
+        programPointer = min(programPointer + 1, len(PROGRAM) - 1)
         numPriorityVehicles = traci.inductionloop.getLastStepVehicleNumber("0")
         if numPriorityVehicles > 0:
-            if programPointer == len(PROGRAM)-1:
-                # we are in the WEGREEN phase. start the priority phase sequence
+            if programPointer == len(PROGRAM) - 1:
+                # we are in the WEGREEN phase. start the priority phase
+                # sequence
                 programPointer = 0
             elif PROGRAM[programPointer] != WEYELLOW:
                 # horizontal traffic is already stopped. restart priority phase
@@ -96,14 +107,17 @@ def run():
             else:
                 # we are in the WEYELLOW phase. continue sequence
                 pass
-        traci.trafficlights.setRedYellowGreenState("0", PROGRAM[programPointer])
+        traci.trafficlights.setRedYellowGreenState(
+            "0", PROGRAM[programPointer])
         step += 1
     traci.close()
     sys.stdout.flush()
 
+
 def get_options():
     optParser = optparse.OptionParser()
-    optParser.add_option("--nogui", action="store_true", default=False, help="run the commandline version of sumo")
+    optParser.add_option("--nogui", action="store_true",
+                         default=False, help="run the commandline version of sumo")
     options, args = optParser.parse_args()
     return options
 
@@ -124,6 +138,7 @@ if __name__ == "__main__":
 
     # this is the normal way of using traci. sumo is started as a
     # subprocess and then the python script connects and runs
-    sumoProcess = subprocess.Popen([sumoBinary, "-c", "data/cross.sumocfg", "--tripinfo-output", "tripinfo.xml", "--remote-port", str(PORT)], stdout=sys.stdout, stderr=sys.stderr)
+    sumoProcess = subprocess.Popen([sumoBinary, "-c", "data/cross.sumocfg", "--tripinfo-output",
+                                    "tripinfo.xml", "--remote-port", str(PORT)], stdout=sys.stdout, stderr=sys.stderr)
     run()
     sumoProcess.wait()
