@@ -5,12 +5,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Michael Behrisch
 /// @date    July 2010
-/// @version $Id: AGPosition.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: AGPosition.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // References a street of the city and defines a position in this street
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 // activitygen module
 // Copyright 2010 TUM (Technische Universitaet Muenchen, http://www.tum.de/)
 /****************************************************************************/
@@ -125,8 +125,8 @@ AGPosition::randomPositionInStreet(const AGStreet& s) {
 Position
 AGPosition::compute2dPosition() const {
     // P = From + pos*(To - From) = pos*To + (1-pos)*From
-    Position From = street->edge->getFromNode()->getPosition();
-    Position To = street->edge->getToNode()->getPosition();
+    Position From = street->getFromNode()->getPosition();
+    Position To = street->getToNode()->getPosition();
     Position position2d(To);
 
     position2d.sub(From);
