@@ -8,12 +8,12 @@
 /// @author  Laura Bieker
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFDetector.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: RODFDetector.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Class representing a detector within the DFROUTER
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2006-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -91,7 +91,7 @@ SUMOReal
 RODFDetector::computeDistanceFactor(const RODFRouteDesc& rd) const {
     SUMOReal distance = rd.edges2Pass[0]->getFromNode()->getPosition().distanceTo(rd.edges2Pass.back()->getToNode()->getPosition());
     SUMOReal length = 0;
-    for (std::vector<ROEdge*>::const_iterator i = rd.edges2Pass.begin(); i != rd.edges2Pass.end(); ++i) {
+    for (ROEdgeVector::const_iterator i = rd.edges2Pass.begin(); i != rd.edges2Pass.end(); ++i) {
         length += (*i)->getLength();
     }
     return (distance / length);
@@ -112,7 +112,7 @@ RODFDetector::computeSplitProbabilities(const RODFNet* net, const RODFDetectorCo
     for (std::vector<RODFRouteDesc>::const_iterator i = routes.begin(); i != routes.end(); ++i) {
         const RODFRouteDesc& rd = *i;
         bool hadSplit = false;
-        for (std::vector<ROEdge*>::const_iterator j = rd.edges2Pass.begin(); j != rd.edges2Pass.end(); ++j) {
+        for (ROEdgeVector::const_iterator j = rd.edges2Pass.begin(); j != rd.edges2Pass.end(); ++j) {
             if (hadSplit && net->hasDetector(*j)) {
                 if (find(nextDetEdges.begin(), nextDetEdges.end(), *j) == nextDetEdges.end()) {
                     nextDetEdges.push_back(static_cast<RODFEdge*>(*j));
@@ -132,15 +132,14 @@ RODFDetector::computeSplitProbabilities(const RODFNet* net, const RODFDetectorCo
     if (OptionsCont::getOptions().getBool("respect-concurrent-inflows")) {
         for (std::vector<RODFEdge*>::const_iterator i = nextDetEdges.begin(); i != nextDetEdges.end(); ++i) {
             std::set<ROEdge*> seen(preSplitEdges);
-            std::vector<ROEdge*> pending;
+            ROEdgeVector pending;
             pending.push_back(*i);
             seen.insert(*i);
             while (!pending.empty()) {
                 ROEdge* e = pending.back();
                 pending.pop_back();
-                const unsigned int numAppr = e->getNumPredecessors();
-                for (unsigned int j = 0; j < numAppr; j++) {
-                    ROEdge* e2 = e->getPredecessor(j);
+                for (ROEdgeVector::const_iterator it = e->getPredecessors().begin(); it != e->getPredecessors().end(); it++) {
+                    ROEdge* e2 = *it;
                     if (e2->getNumSuccessors() == 1 && seen.count(e2) == 0) {
                         if (net->hasDetector(e2)) {
                             inFlows[*i] += detectors.getAggFlowFor(e2, 0, 0, flows);
@@ -194,7 +193,7 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
         size_t index = 0;
         for (std::vector<RODFRouteDesc>::iterator ri = descs.begin(); ri != descs.end(); ++ri, index++) {
             SUMOReal prob = 1.;
-            for (std::vector<ROEdge*>::iterator j = (*ri).edges2Pass.begin(); j != (*ri).edges2Pass.end() && prob > 0;) {
+            for (ROEdgeVector::iterator j = (*ri).edges2Pass.begin(); j != (*ri).edges2Pass.end() && prob > 0;) {
                 if (!net.hasDetector(*j)) {
                     ++j;
                     continue;
