@@ -5,12 +5,12 @@
 /// @author  Christian Roessel
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIApplicationWindow.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIApplicationWindow.h 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // The main window of the SUMO-gui.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -134,6 +134,9 @@ public:
     /// @brief Called on menu File->Open Network
     long onCmdOpenNetwork(FXObject*, FXSelector, void*);
 
+    /// @brief Called on menu File->Load Shapes
+    long onCmdOpenShapes(FXObject*, FXSelector, void*);
+
     /// @brief Called on reload
     long onCmdReload(FXObject*, FXSelector, void*);
 
@@ -160,6 +163,9 @@ public:
     /// @brief Toggle gaming mode
     long onCmdGaming(FXObject*, FXSelector, void*);
 
+    /// @brief Toggle full screen mode
+    long onCmdFullScreen(FXObject*, FXSelector, void*);
+
     /// @brief Toggle listing of internal structures
     long onCmdListInternal(FXObject*, FXSelector, void*);
 
@@ -229,6 +235,14 @@ public:
     long onClipboardRequest(FXObject* sender, FXSelector sel, void* ptr);
     /// @}
 
+
+    /** @brief Returns the simulation delay
+     * @return delay in milliseconds
+     */
+    virtual SUMOReal getDelay() const {
+        return mySimDelayTarget->getValue();
+    }
+
 protected:
     virtual void addToWindowsMenu(FXMenuPane*) { }
 
@@ -334,6 +348,11 @@ protected:
     /// @brief whether to show time as hour:minute:second
     bool myShowTimeAsHMS;
 
+    /// @brief whether to show the window in full screen mode
+    bool myAmFullScreen;
+
+    /// @brief whether the simulation end was already announced
+    bool myHaveNotifiedAboutSimEnd;
 
     /// @name game related things
     /// {
