@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Oct/Nov 2003
-/// @version $Id: GUITLLogicPhasesTrackerWindow.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUITLLogicPhasesTrackerWindow.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // A window displaying the phase diagram of a tl-logic
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -254,6 +254,10 @@ GUITLLogicPhasesTrackerWindow::drawValues(GUITLLogicPhasesTrackerPanel& caller)
             myDurations.push_back((*j)->duration);
             myLastTime += (*j)->duration;
         }
+        if (myLastTime <= myBeginTime) {
+            WRITE_ERROR("Overflow in time computation occured.");
+            return;
+        }
     } else {
         SUMOTime beginOffset = TIME2STEPS(myBeginOffset->getValue());
         myBeginTime = myLastTime - beginOffset;
