@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 13.06.2012
-/// @version $Id: GUIPersonControl.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIPersonControl.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // GUI-version of the person control for building gui persons
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -57,4 +57,13 @@ GUIPersonControl::buildPerson(const SUMOVehicleParameter* pars, const MSVehicleT
 }
 
 
+void
+GUIPersonControl::insertPersonIDs(std::vector<GUIGlID>& into) {
+    into.reserve(myPersons.size());
+    for (std::map<std::string, MSPerson*>::const_iterator it = myPersons.begin(); it != myPersons.end(); ++it) {
+        into.push_back(static_cast<const GUIPerson*>((*it).second)->getGlID());
+    }
+}
+
+
 /****************************************************************************/
