@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 13.06.2012
-/// @version $Id: GUIPersonControl.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIPersonControl.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // GUI-version of the person control for building gui persons
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -59,6 +59,11 @@ public:
      */
     virtual MSPerson* buildPerson(const SUMOVehicleParameter* pars, const MSVehicleType* vtype, MSPerson::MSPersonPlan* plan) const;
 
+    /** @brief Returns the list of all known persons by gl-id
+     * @param[fill] into The list to fill with vehicle ids
+     * @todo Well, what about concurrent modifications?
+     */
+    void insertPersonIDs(std::vector<GUIGlID>& into);
 };
 
 
