@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: MSRouteHandler.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSRouteHandler.cpp 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // Parser and container for routes during their loading
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -44,6 +44,7 @@
 #include <microsim/MSLane.h>
 #include "MSRouteHandler.h"
 #include "MSPersonControl.h"
+#include "MSContainerControl.h"
 #include <utils/xml/SUMOSAXHandler.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
 #include <utils/common/MsgHandler.h>
@@ -54,6 +55,7 @@
 #include "MSNet.h"
 
 #include <microsim/trigger/MSBusStop.h>
+//#include <microsim/trigger/MSContainerStop.h>
 #include <microsim/MSGlobals.h>
 #include <utils/xml/SUMOVehicleParserHelper.h>
 
@@ -75,6 +77,7 @@ MSRouteHandler::MSRouteHandler(const std::string& file,
                                bool addVehiclesDirectly) :
     SUMORouteHandler(file),
     myActivePlan(0),
+    myActiveContainerPlan(0),
     myAddVehiclesDirectly(addVehiclesDirectly),
     myCurrentVTypeDistribution(0),
     myCurrentRouteDistribution(0) {
@@ -86,6 +89,54 @@ MSRouteHandler::~MSRouteHandler() {
 }
 
 
+void
+MSRouteHandler::parseFromViaTo(std::string element,
+                               const SUMOSAXAttributes& attrs) {
+    myActiveRoute.clear();
+    bool useTaz = OptionsCont::getOptions().getBool("with-taz");
+    if (useTaz && !myVehicleParameter->wasSet(VEHPARS_FROM_TAZ_SET) && !myVehicleParameter->wasSet(VEHPARS_TO_TAZ_SET)) {
+        WRITE_WARNING("Taz usage was requested but no taz present in " + element + " '" + myVehicleParameter->id + "'!");
+        useTaz = false;
+    }
+    bool ok = true;
+    if ((useTaz || !attrs.hasAttribute(SUMO_ATTR_FROM)) && myVehicleParameter->wasSet(VEHPARS_FROM_TAZ_SET)) {
+        const MSEdge* fromTaz = MSEdge::dictionary(myVehicleParameter->fromTaz + "-source");
+        if (fromTaz == 0) {
+            throw ProcessError("Source taz '" + myVehicleParameter->fromTaz + "' not known for " + element + " '" + myVehicleParameter->id + "'!");
+        } else if (fromTaz->getNumSuccessors() == 0) {
+            throw ProcessError("Source taz '" + myVehicleParameter->fromTaz + "' has no outgoing edges for " + element + " '" + myVehicleParameter->id + "'!");
+        } else {
+            myActiveRoute.push_back(fromTaz);
+        }
+    } else {
+        MSEdge::parseEdgesList(attrs.getOpt<std::string>(SUMO_ATTR_FROM, myVehicleParameter->id.c_str(), ok, "", true),
+                               myActiveRoute, "for " + element + " '" + myVehicleParameter->id + "'");
+    }
+    if (!attrs.hasAttribute(SUMO_ATTR_VIA) && !attrs.hasAttribute(SUMO_ATTR_ROUTE)) {
+        myInsertStopEdgesAt = (int)myActiveRoute.size();
+    }
+    MSEdge::parseEdgesList(attrs.getOpt<std::string>(SUMO_ATTR_VIA, myVehicleParameter->id.c_str(), ok, "", true),
+                           myActiveRoute, "for " + element + " '" + myVehicleParameter->id + "'");
+    if ((useTaz || !attrs.hasAttribute(SUMO_ATTR_TO)) && myVehicleParameter->wasSet(VEHPARS_TO_TAZ_SET)) {
+        const MSEdge* toTaz = MSEdge::dictionary(myVehicleParameter->toTaz + "-sink");
+        if (toTaz == 0) {
+            throw ProcessError("Sink taz '" + myVehicleParameter->toTaz + "' not known for " + element + " '" + myVehicleParameter->id + "'!");
+        } else if (toTaz->getNumPredecessors() == 0) {
+            throw ProcessError("Sink taz '" + myVehicleParameter->toTaz + "' has no incoming edges for " + element + " '" + myVehicleParameter->id + "'!");
+        } else {
+            myActiveRoute.push_back(toTaz);
+        }
+    } else {
+        MSEdge::parseEdgesList(attrs.getOpt<std::string>(SUMO_ATTR_TO, myVehicleParameter->id.c_str(), ok, "", true),
+                               myActiveRoute, "for " + element + " '" + myVehicleParameter->id + "'");
+    }
+    myActiveRouteID = "!" + myVehicleParameter->id;
+    if (myVehicleParameter->routeid == "") {
+        myVehicleParameter->routeid = myActiveRouteID;
+    }
+}
+
+
 void
 MSRouteHandler::myStartElement(int element,
                                const SUMOSAXAttributes& attrs) {
@@ -94,6 +145,9 @@ MSRouteHandler::myStartElement(int element,
         case SUMO_TAG_PERSON:
             myActivePlan = new MSPerson::MSPersonPlan();
             break;
+        case SUMO_TAG_CONTAINER:
+            myActiveContainerPlan = new MSContainer::MSContainerPlan();
+            break;
         case SUMO_TAG_RIDE: {
             const std::string pid = myVehicleParameter->id;
             bool ok = true;
@@ -182,8 +236,7 @@ MSRouteHandler::myStartElement(int element,
                         const std::string error = "No connection found between '" + from->getID() + "' and '" + to->getID() + "' for person '" + myVehicleParameter->id + "'.";
                         if (OptionsCont::getOptions().getBool("ignore-route-errors")) {
                             myActiveRoute.push_back(from);
-                            // XXX
-                            //myActiveRoute.push_back(to);
+                            myActiveRoute.push_back(to); // pedestrian will teleport
                             //WRITE_WARNING(error);
                         } else {
                             WRITE_ERROR(error);
@@ -206,39 +259,115 @@ MSRouteHandler::myStartElement(int element,
             myActiveRoute.clear();
             break;
         }
-        case SUMO_TAG_FLOW:
-            if (attrs.hasAttribute(SUMO_ATTR_FROM) && attrs.hasAttribute(SUMO_ATTR_TO)) {
-                myActiveRouteID = "!" + myVehicleParameter->id;
-                bool ok = true;
-                MSEdge::parseEdgesList(attrs.get<std::string>(SUMO_ATTR_FROM, myVehicleParameter->id.c_str(), ok),
-                                       myActiveRoute, "for vehicle '" + myVehicleParameter->id + "'");
-                MSEdge::parseEdgesList(attrs.get<std::string>(SUMO_ATTR_TO, myVehicleParameter->id.c_str(), ok),
-                                       myActiveRoute, "for vehicle '" + myVehicleParameter->id + "'");
-                closeRoute(true);
-                myVehicleParameter->setParameter |= VEHPARS_FORCE_REROUTE;
-            }
-            break;
-        case SUMO_TAG_TRIP: {
+        case SUMO_TAG_TRANSPORT: {
+            const std::string containerId = myVehicleParameter->id;
             bool ok = true;
-            if (attrs.hasAttribute(SUMO_ATTR_FROM) || !myVehicleParameter->wasSet(VEHPARS_TAZ_SET)) {
-                MSEdge::parseEdgesList(attrs.get<std::string>(SUMO_ATTR_FROM, myVehicleParameter->id.c_str(), ok),
-                                       myActiveRoute, "for vehicle '" + myVehicleParameter->id + "'");
-                MSEdge::parseEdgesList(attrs.get<std::string>(SUMO_ATTR_TO, myVehicleParameter->id.c_str(), ok),
-                                       myActiveRoute, "for vehicle '" + myVehicleParameter->id + "'");
-            } else {
-                const MSEdge* fromTaz = MSEdge::dictionary(myVehicleParameter->fromTaz + "-source");
-                if (fromTaz == 0) {
-                    WRITE_ERROR("Source district '" + myVehicleParameter->fromTaz + "' not known for '" + myVehicleParameter->id + "'!");
-                } else if (fromTaz->getNumSuccessors() == 0) {
-                    WRITE_ERROR("Source district '" + myVehicleParameter->fromTaz + "' has no outgoing edges for '" + myVehicleParameter->id + "'!");
-                } else {
-                    myActiveRoute.push_back(fromTaz->getSuccessor(0));
+            MSEdge* from = 0;
+            const std::string desc = attrs.get<std::string>(SUMO_ATTR_LINES, containerId.c_str(), ok);
+            StringTokenizer st(desc);
+            std::string csID = attrs.getOpt<std::string>(SUMO_ATTR_CONTAINER_STOP, 0, ok, "");
+            MSContainerStop* cs = 0;
+            if (csID != "") {
+                cs = MSNet::getInstance()->getContainerStop(csID);
+                if (cs == 0) {
+                    throw ProcessError("Unknown container stop '" + csID + "' for container '" + myVehicleParameter->id + "'.");
                 }
             }
-            closeRoute(true);
-            closeVehicle();
+            if (attrs.hasAttribute(SUMO_ATTR_FROM)) {
+                const std::string fromID = attrs.get<std::string>(SUMO_ATTR_FROM, containerId.c_str(), ok);
+                from = MSEdge::dictionary(fromID);
+                if (from == 0) {
+                    throw ProcessError("The from edge '" + fromID + "' within a transport of container '" + containerId + "' is not known.");
+                }
+                if (!myActiveContainerPlan->empty() && &myActiveContainerPlan->back()->getDestination() != from) {
+                    throw ProcessError("Disconnected plan for container '" + myVehicleParameter->id + "' (" + fromID + "!=" + myActiveContainerPlan->back()->getDestination().getID() + ").");
+                }
+                if (myActiveContainerPlan->empty()) {
+                    myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Waiting(
+                                                         *from, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start"));
+                }
+            } else if (myActiveContainerPlan->empty()) {
+                throw ProcessError("The start edge within a transport of container '" + containerId + "' is not known.");
+            }
+            const std::string toID = attrs.get<std::string>(SUMO_ATTR_TO, containerId.c_str(), ok);
+            MSEdge* to = MSEdge::dictionary(toID);
+            if (to == 0) {
+                throw ProcessError("The to edge '" + toID + "' within a transport of container '" + containerId + "' is not known.");
+            }
+            myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Driving(*to, cs, st.getVector()));
+            break;
         }
-        break;
+        case SUMO_TAG_TRANSHIP: {
+            myActiveRoute.clear();
+            bool ok = true;
+            SUMOReal departPos = attrs.getOpt<SUMOReal>(SUMO_ATTR_DEPARTPOS, myVehicleParameter->id.c_str(), ok, 0);
+            SUMOReal arrivalPos = attrs.getOpt<SUMOReal>(SUMO_ATTR_ARRIVALPOS, myVehicleParameter->id.c_str(), ok, -NUMERICAL_EPS);
+            SUMOReal speed = DEFAULT_CONTAINER_TRANSHIP_SPEED;
+            const MSVehicleType* vtype = MSNet::getInstance()->getVehicleControl().getVType(myVehicleParameter->vtypeid);
+            // need to check for explicitly set speed since we might have // DEFAULT_VEHTYPE
+            if (vtype != 0 && vtype->wasSet(VTYPEPARS_MAXSPEED_SET)) {
+                speed = vtype->getMaxSpeed();
+            }
+            speed = attrs.getOpt<SUMOReal>(SUMO_ATTR_SPEED, 0, ok, speed);
+            if (speed <= 0) {
+                throw ProcessError("Non-positive tranship speed for container  '" + myVehicleParameter->id + "'.");
+            }
+            std::string csID = attrs.getOpt<std::string>(SUMO_ATTR_CONTAINER_STOP, 0, ok, "");
+            MSContainerStop* cs = 0;
+            if (csID != "") {
+                cs = MSNet::getInstance()->getContainerStop(csID);
+                if (cs == 0) {
+                    throw ProcessError("Unknown container stop '" + csID + "' for container '" + myVehicleParameter->id + "'.");
+                }
+                arrivalPos = cs->getEndLanePosition();
+            }
+            if (attrs.hasAttribute(SUMO_ATTR_EDGES)) {
+                MSEdge::parseEdgesList(attrs.get<std::string>(SUMO_ATTR_EDGES, myVehicleParameter->id.c_str(), ok), myActiveRoute, myActiveRouteID);
+            } else {
+                if (attrs.hasAttribute(SUMO_ATTR_FROM) && attrs.hasAttribute(SUMO_ATTR_TO)) {
+                    const std::string fromID = attrs.get<std::string>(SUMO_ATTR_FROM, myVehicleParameter->id.c_str(), ok);
+                    MSEdge* from = MSEdge::dictionary(fromID);
+                    if (from == 0) {
+                        throw ProcessError("The from edge '" + fromID + "' within a tranship of container '" + myVehicleParameter->id + "' is not known.");
+                    }
+                    const std::string toID = attrs.get<std::string>(SUMO_ATTR_TO, myVehicleParameter->id.c_str(), ok);
+                    MSEdge* to = MSEdge::dictionary(toID);
+                    if (to == 0) {
+                        throw ProcessError("The to edge '" + toID + "' within a tranship of container '" + myVehicleParameter->id + "' is not known.");
+                    }
+                    //the route of the container's tranship stage consists only of the 'from' and the 'to' edge
+                    myActiveRoute.push_back(from);
+                    myActiveRoute.push_back(to);
+                    if (myActiveRoute.empty()) {
+                        const std::string error = "No connection found between '" + from->getID() + "' and '" + to->getID() + "' for container '" + myVehicleParameter->id + "'.";
+                        if (OptionsCont::getOptions().getBool("ignore-route-errors")) {
+                            myActiveRoute.push_back(from);
+                        } else {
+                            WRITE_ERROR(error);
+                        }
+                    }
+                }
+            }
+            if (myActiveRoute.empty()) {
+                throw ProcessError("No edges to tranship container '" + myVehicleParameter->id + "'.");
+            }
+            if (!myActiveContainerPlan->empty() && &myActiveContainerPlan->back()->getDestination() != myActiveRoute.front()) {
+                throw ProcessError("Disconnected plan for container '" + myVehicleParameter->id + "' (" + myActiveRoute.front()->getID() + "!=" + myActiveContainerPlan->back()->getDestination().getID() + ").");
+            }
+            if (myActiveContainerPlan->empty()) {
+                myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Waiting(
+                                                     *myActiveRoute.front(), -1, myVehicleParameter->depart, departPos, "start"));
+            }
+            myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Tranship(myActiveRoute, cs, speed, departPos, arrivalPos));
+            myActiveRoute.clear();
+            break;
+        }
+        case SUMO_TAG_FLOW:
+            parseFromViaTo("flow", attrs);
+            break;
+        case SUMO_TAG_TRIP:
+            parseFromViaTo("trip", attrs);
+            break;
         default:
             break;
     }
@@ -294,6 +423,8 @@ MSRouteHandler::closeVehicleTypeDistribution() {
 
 void
 MSRouteHandler::openRoute(const SUMOSAXAttributes& attrs) {
+    myActiveRoute.clear();
+    myInsertStopEdgesAt = -1;
     // check whether the id is really necessary
     std::string rid;
     if (myCurrentRouteDistribution != 0) {
@@ -355,6 +486,14 @@ MSRouteHandler::myEndElement(int element) {
             }
         }
         break;
+        case SUMO_TAG_TRIP:
+            myVehicleParameter->setParameter |= VEHPARS_FORCE_REROUTE;
+            closeRoute(true);
+            closeVehicle();
+            delete myVehicleParameter;
+            myVehicleParameter = 0;
+            myInsertStopEdgesAt = -1;
+            break;
         default:
             break;
     }
@@ -362,7 +501,15 @@ MSRouteHandler::myEndElement(int element) {
 
 
 void
-MSRouteHandler::closeRoute(const bool /* mayBeDisconnected */) {
+MSRouteHandler::closeRoute(const bool mayBeDisconnected) {
+    std::string type = "vehicle";
+    if (mayBeDisconnected) {
+        if (myVehicleParameter->repetitionNumber >= 0) {
+            type = "flow";
+        } else {
+            type = "trip";
+        }
+    }
     if (myActiveRoute.size() == 0) {
         delete myActiveRouteColor;
         myActiveRouteColor = 0;
@@ -378,11 +525,14 @@ MSRouteHandler::closeRoute(const bool /* mayBeDisconnected */) {
             return;
         }
         if (myVehicleParameter != 0) {
-            throw ProcessError("Vehicle's '" + myVehicleParameter->id + "' route has no edges.");
+            throw ProcessError("The route for " + type + " '" + myVehicleParameter->id + "' has no edges.");
         } else {
             throw ProcessError("Route '" + myActiveRouteID + "' has no edges.");
         }
     }
+    if (myActiveRoute.size() == 1 && myActiveRoute.front()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT) {
+        throw ProcessError("The routing information for " + type + " '" + myVehicleParameter->id + "' is insufficient.");
+    }
     MSRoute* route = new MSRoute(myActiveRouteID, myActiveRoute,
                                  myVehicleParameter == 0 || myVehicleParameter->repetitionNumber >= 1,
                                  myActiveRouteColor, myActiveRouteStops);
@@ -393,7 +543,7 @@ MSRouteHandler::closeRoute(const bool /* mayBeDisconnected */) {
         if (!MSGlobals::gStateLoaded) {
             if (myVehicleParameter != 0) {
                 if (MSNet::getInstance()->getVehicleControl().getVehicle(myVehicleParameter->id) == 0) {
-                    throw ProcessError("Another route for vehicle '" + myVehicleParameter->id + "' exists.");
+                    throw ProcessError("Another route for " + type + " '" + myVehicleParameter->id + "' exists.");
                 } else {
                     throw ProcessError("A vehicle with id '" + myVehicleParameter->id + "' already exists.");
                 }
@@ -526,14 +676,34 @@ MSRouteHandler::closeVehicle() {
     // try to build the vehicle
     SUMOVehicle* vehicle = 0;
     if (vehControl.getVehicle(myVehicleParameter->id) == 0) {
-        vehicle = vehControl.buildVehicle(myVehicleParameter, route, vtype, OptionsCont::getOptions().getBool("ignore-route-errors"));
+        const bool ignoreRouteErrors = OptionsCont::getOptions().getBool("ignore-route-errors");
+        try {
+            vehicle = vehControl.buildVehicle(myVehicleParameter, route, vtype, ignoreRouteErrors);
+        } catch (const ProcessError& e) {
+            if (ignoreRouteErrors) {
+                WRITE_WARNING(e.what());
+                vehControl.deleteVehicle(0, true);
+                myVehicleParameter = 0;
+                vehicle = 0;
+                return;
+            } else {
+                throw e;
+            }
+        }
         // maybe we do not want this vehicle to be inserted due to scaling
         unsigned int quota = vehControl.getQuota();
         if (quota > 0) {
             vehControl.addVehicle(myVehicleParameter->id, vehicle);
             if (myVehicleParameter->departProcedure == DEPART_TRIGGERED) {
+                const MSEdge* const firstEdge = vehicle->getRoute().getEdges()[0];
+                static_cast<MSVehicle*>(vehicle)->setTentativeLaneAndPosition(firstEdge->getLanes()[0], myVehicleParameter->departPos);
                 vehControl.addWaiting(*route->begin(), vehicle);
                 vehControl.registerOneWaitingForPerson();
+            } else if (myVehicleParameter->departProcedure == DEPART_CONTAINER_TRIGGERED) {
+                const MSEdge* const firstEdge = vehicle->getRoute().getEdges()[0];
+                static_cast<MSVehicle*>(vehicle)->setTentativeLaneAndPosition(firstEdge->getLanes()[0], myVehicleParameter->departPos);
+                vehControl.addWaiting(*route->begin(), vehicle);
+                vehControl.registerOneWaitingForContainer();
             } else {
                 // !!! no scaling for triggered vehicles yet
                 for (unsigned int i = 1; i < quota; i++) {
@@ -588,8 +758,9 @@ MSRouteHandler::closePerson() {
             MSNet::getInstance()->getPersonControl().setDeparture(myVehicleParameter->depart, person);
             registerLastDepart();
         } else {
+            ProcessError error("Another person with the id '" + myVehicleParameter->id + "' exists.");
             delete person;
-            throw ProcessError("Another person with the id '" + myVehicleParameter->id + "' exists.");
+            throw error;
         }
     } else {
         // warning already given
@@ -599,6 +770,33 @@ MSRouteHandler::closePerson() {
     myActivePlan = 0;
 }
 
+void
+MSRouteHandler::closeContainer() {
+    if (myActiveContainerPlan->size() == 0) {
+        throw ProcessError("Container '" + myVehicleParameter->id + "' has no plan.");
+    }
+    MSVehicleType* type = MSNet::getInstance()->getVehicleControl().getVType(myVehicleParameter->vtypeid);
+    if (type == 0) {
+        throw ProcessError("The type '" + myVehicleParameter->vtypeid + "' for container '" + myVehicleParameter->id + "' is not known.");
+    }
+    MSContainer* container = MSNet::getInstance()->getContainerControl().buildContainer(myVehicleParameter, type, myActiveContainerPlan);
+    // @todo: consider myScale?
+    if (myAddVehiclesDirectly || checkLastDepart()) {
+        if (MSNet::getInstance()->getContainerControl().add(myVehicleParameter->id, container)) {
+            MSNet::getInstance()->getContainerControl().setDeparture(myVehicleParameter->depart, container);
+            registerLastDepart();
+        } else {
+            ProcessError error("Another container with the id '" + myVehicleParameter->id + "' exists.");
+            delete container;
+            throw error;
+        }
+    } else {
+        // warning already given
+        delete container;
+    }
+    myVehicleParameter = 0;
+    myActiveContainerPlan = 0;
+}
 
 void
 MSRouteHandler::closeFlow() {
@@ -619,72 +817,73 @@ MSRouteHandler::closeFlow() {
     if (MSNet::getInstance()->getVehicleControl().getVType(myVehicleParameter->vtypeid, &myParsingRNG) == 0) {
         throw ProcessError("The vehicle type '" + myVehicleParameter->vtypeid + "' for flow '" + myVehicleParameter->id + "' is not known.");
     }
-    if (MSRoute::dictionary("!" + myVehicleParameter->id, &myParsingRNG) == 0) {
-        // if not, try via the (hopefully) given route-id
-        if (MSRoute::dictionary(myVehicleParameter->routeid, &myParsingRNG) == 0) {
-            if (myVehicleParameter->routeid != "") {
-                throw ProcessError("The route '" + myVehicleParameter->routeid + "' for flow '" + myVehicleParameter->id + "' is not known.");
-            } else {
-                if (myVehicleParameter->wasSet(VEHPARS_TAZ_SET)) {
-                    myVehicleParameter->setParameter |= VEHPARS_FORCE_REROUTE;
-                    const MSEdge* fromTaz = MSEdge::dictionary(myVehicleParameter->fromTaz + "-source");
-                    if (fromTaz == 0) {
-                        WRITE_ERROR("Source district '" + myVehicleParameter->fromTaz + "' not known for '" + myVehicleParameter->id + "'!");
-                    } else if (fromTaz->getNumSuccessors() == 0) {
-                        WRITE_ERROR("Source district '" + myVehicleParameter->fromTaz + "' has no outgoing edges for '" + myVehicleParameter->id + "'!");
-                    } else {
-                        myActiveRoute.push_back(fromTaz->getSuccessor(0));
-                    }
-                    closeRoute(true);
-                    myVehicleParameter->routeid = "!" + myVehicleParameter->id;
-                } else {
-                    throw ProcessError("Flow '" + myVehicleParameter->id + "' has no route.");
-                }
-            }
-        }
-    } else {
-        myVehicleParameter->routeid = "!" + myVehicleParameter->id;
+    if (myVehicleParameter->routeid[0] == '!' && MSRoute::dictionary(myVehicleParameter->routeid, &myParsingRNG) == 0) {
+        myVehicleParameter->setParameter |= VEHPARS_FORCE_REROUTE;
+        closeRoute(true);
+    }
+    if (MSRoute::dictionary(myVehicleParameter->routeid, &myParsingRNG) == 0) {
+        throw ProcessError("The route '" + myVehicleParameter->routeid + "' for flow '" + myVehicleParameter->id + "' is not known.");
     }
     myActiveRouteID = "";
 
     // check whether the vehicle shall be added directly to the network or
     //  shall stay in the internal buffer
     if (myAddVehiclesDirectly || checkLastDepart()) {
-        MSNet::getInstance()->getInsertionControl().add(myVehicleParameter);
-        registerLastDepart();
+        if (MSNet::getInstance()->getInsertionControl().add(myVehicleParameter)) {
+            registerLastDepart();
+        } else {
+            throw ProcessError("Another flow with the id '" + myVehicleParameter->id + "' exists.");
+        }
     }
     myVehicleParameter = 0;
+    myInsertStopEdgesAt = -1;
 }
 
 
 void
 MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
     std::string errorSuffix;
-    if (myActiveRouteID != "") {
-        errorSuffix = " in route '" + myActiveRouteID + "'.";
-    } else if (myActivePlan) {
+    if (myActivePlan != 0) {
         errorSuffix = " in person '" + myVehicleParameter->id + "'.";
-    } else {
+    } else if (myVehicleParameter != 0) {
         errorSuffix = " in vehicle '" + myVehicleParameter->id + "'.";
+    } else if (myActiveContainerPlan != 0) {
+        errorSuffix = " in container '" + myVehicleParameter->id + "'.";
+    } else {
+        errorSuffix = " in route '" + myActiveRouteID + "'.";
     }
     SUMOVehicleParameter::Stop stop;
     bool ok = parseStop(stop, attrs, errorSuffix, MsgHandler::getErrorInstance());
     if (!ok) {
         return;
     }
+    MSEdge* edge = 0;
     // try to parse the assigned bus stop
     if (stop.busstop != "") {
         // ok, we have obviously a bus stop
         MSBusStop* bs = MSNet::getInstance()->getBusStop(stop.busstop);
-        if (bs != 0) {
-            const MSLane& l = bs->getLane();
-            stop.lane = l.getID();
-            stop.endPos = bs->getEndLanePosition();
-            stop.startPos = bs->getBeginLanePosition();
-        } else {
+        if (bs == 0) {
             WRITE_ERROR("The bus stop '" + stop.busstop + "' is not known" + errorSuffix);
             return;
         }
+        const MSLane& l = bs->getLane();
+        stop.lane = l.getID();
+        stop.endPos = bs->getEndLanePosition();
+        stop.startPos = bs->getBeginLanePosition();
+        edge = &l.getEdge();
+    } //try to parse the assigned container stop
+    else if (stop.containerstop != "") {
+        // ok, we have obviously a container stop
+        MSContainerStop* cs = MSNet::getInstance()->getContainerStop(stop.containerstop);
+        if (cs == 0) {
+            WRITE_ERROR("The container stop '" + stop.containerstop + "' is not known" + errorSuffix);
+            return;
+        }
+        const MSLane& l = cs->getLane();
+        stop.lane = l.getID();
+        stop.endPos = cs->getEndLanePosition();
+        stop.startPos = cs->getBeginLanePosition();
+        edge = &l.getEdge();
     } else {
         // no, the lane and the position should be given
         // get the lane
@@ -695,17 +894,27 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
                 return;
             }
         } else {
-            WRITE_ERROR("A stop must be placed on a bus stop or a lane" + errorSuffix);
+            WRITE_ERROR("A stop must be placed on a bus stop, a container stop or a lane" + errorSuffix);
             return;
         }
+        edge = &MSLane::dictionary(stop.lane)->getEdge();
         if (myActivePlan &&
                 !myActivePlan->empty() &&
-                &myActivePlan->back()->getDestination() != &MSLane::dictionary(stop.lane)->getEdge()) {
-            throw ProcessError("Disconnected plan for person '" + myVehicleParameter->id + "' (" + MSLane::dictionary(stop.lane)->getEdge().getID() + "!=" + myActivePlan->back()->getDestination().getID() + ").");
+                &myActivePlan->back()->getDestination() != edge) {
+            throw ProcessError("Disconnected plan for person '" + myVehicleParameter->id + "' (" + edge->getID() + "!=" + myActivePlan->back()->getDestination().getID() + ").");
         }
         if (myActivePlan && myActivePlan->empty()) {
             myActivePlan->push_back(new MSPerson::MSPersonStage_Waiting(
-                                        MSLane::dictionary(stop.lane)->getEdge(), -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start"));
+                                        *edge, -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start"));
+        }
+        if (myActiveContainerPlan &&
+                !myActiveContainerPlan->empty() &&
+                &myActiveContainerPlan->back()->getDestination() != &MSLane::dictionary(stop.lane)->getEdge()) {
+            throw ProcessError("Disconnected plan for container '" + myVehicleParameter->id + "' (" + MSLane::dictionary(stop.lane)->getEdge().getID() + "!=" + myActiveContainerPlan->back()->getDestination().getID() + ").");
+        }
+        if (myActiveContainerPlan && myActiveContainerPlan->empty()) {
+            myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Waiting(
+                                                 MSLane::dictionary(stop.lane)->getEdge(), -1, myVehicleParameter->depart, myVehicleParameter->departPos, "start"));
         }
         stop.endPos = attrs.getOpt<SUMOReal>(SUMO_ATTR_ENDPOS, 0, ok, MSLane::dictionary(stop.lane)->getLength());
         if (attrs.hasAttribute(SUMO_ATTR_POSITION)) {
@@ -719,14 +928,22 @@ MSRouteHandler::addStop(const SUMOSAXAttributes& attrs) {
             return;
         }
     }
-    if (myActiveRouteID != "") {
-        myActiveRouteStops.push_back(stop);
-    } else if (myActivePlan) {
+    if (myActivePlan != 0) {
         std::string actType = attrs.getOpt<std::string>(SUMO_ATTR_ACTTYPE, 0, ok, "waiting");
         myActivePlan->push_back(new MSPerson::MSPersonStage_Waiting(
                                     MSLane::dictionary(stop.lane)->getEdge(), stop.duration, stop.until, stop.startPos, actType));
-    } else {
+    } else if (myActiveContainerPlan != 0) {
+        std::string actType = attrs.getOpt<std::string>(SUMO_ATTR_ACTTYPE, 0, ok, "waiting");
+        myActiveContainerPlan->push_back(new MSContainer::MSContainerStage_Waiting(
+                                             MSLane::dictionary(stop.lane)->getEdge(), stop.duration, stop.until, stop.startPos, actType));
+    } else if (myVehicleParameter != 0) {
         myVehicleParameter->stops.push_back(stop);
+    } else {
+        myActiveRouteStops.push_back(stop);
+    }
+    if (myInsertStopEdgesAt >= 0) {
+        myActiveRoute.insert(myActiveRoute.begin() + myInsertStopEdgesAt, edge);
+        myInsertStopEdgesAt++;
     }
 }
 
