@@ -4,12 +4,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Tue, 04 Dec 2007
-/// @version $Id: MSDevice_Routing.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: MSDevice_Routing.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // A device that performs vehicle rerouting based on current edge speeds
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2007-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2007-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -325,6 +325,9 @@ private:
     /// @brief The router to use
     static SUMOAbstractRouter<MSEdge, SUMOVehicle>* myRouter;
 
+    /// @brief Whether to disturb edge weights dynamically
+    static SUMOReal myRandomizeWeightsFactor;
+
 #ifdef HAVE_FOX
     static FXWorkerThread::Pool myThreadPool;
 #endif
