@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    2012-04-26
-/// @version $Id: MSEmissionExport.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSEmissionExport.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Realises dumping Emission Data
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -33,6 +33,7 @@
 
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/emissions/PollutantsInterface.h>
+#include <utils/emissions/HelpersHarmonoise.h>
 #include <microsim/MSNet.h>
 #include <microsim/MSVehicle.h>
 #include "MSEmissionExport.h"
@@ -47,29 +48,28 @@
 // ===========================================================================
 void
 MSEmissionExport::write(OutputDevice& of, SUMOTime timestep) {
-
     of.openTag("timestep").writeAttr("time", time2string(timestep));
-
     MSVehicleControl& vc = MSNet::getInstance()->getVehicleControl();
-    MSVehicleControl::constVehIt it = vc.loadedVehBegin();
-    MSVehicleControl::constVehIt end = vc.loadedVehEnd();
-    for (; it != end; ++it) {
-        const MSVehicle* veh = static_cast<const MSVehicle*>((*it).second);
-        if (!veh->isOnRoad()) {
-            continue;
+    for (MSVehicleControl::constVehIt it = vc.loadedVehBegin(); it != vc.loadedVehEnd(); ++it) {
+        const SUMOVehicle* veh = it->second;
+        const MSVehicle* microVeh = dynamic_cast<const MSVehicle*>(veh);
+        if (veh->isOnRoad()) {
+            std::string fclass = veh->getVehicleType().getID();
+            fclass = fclass.substr(0, fclass.find_first_of("@"));
+            PollutantsInterface::Emissions emiss = PollutantsInterface::computeAll(veh->getVehicleType().getEmissionClass(), veh->getSpeed(), veh->getAcceleration(), veh->getSlope());
+            of.openTag("vehicle").writeAttr("id", veh->getID()).writeAttr("eclass", PollutantsInterface::getName(veh->getVehicleType().getEmissionClass()));
+            of.writeAttr("CO2", emiss.CO2).writeAttr("CO", emiss.CO).writeAttr("HC", emiss.HC).writeAttr("NOx", emiss.NOx);
+            of.writeAttr("PMx", emiss.PMx).writeAttr("fuel", emiss.fuel);
+            of.writeAttr("noise", HelpersHarmonoise::computeNoise(veh->getVehicleType().getEmissionClass(), veh->getSpeed(), veh->getAcceleration()));
+            of.writeAttr("route", veh->getRoute().getID()).writeAttr("type", fclass);
+            if (microVeh != 0) {
+                of.writeAttr("waiting", microVeh->getWaitingSeconds());
+                of.writeAttr("lane", microVeh->getLane()->getID());
+            }
+            of.writeAttr("pos", veh->getPositionOnLane()).writeAttr("speed", veh->getSpeed());
+            of.writeAttr("angle", veh->getAngle()).writeAttr("x", veh->getPosition().x()).writeAttr("y", veh->getPosition().y());
+            of.closeTag();
         }
-
-        std::string fclass = veh->getVehicleType().getID();
-        fclass = fclass.substr(0, fclass.find_first_of("@"));
-
-        Position pos = veh->getLane()->getShape().positionAtOffset(veh->getPositionOnLane());
-        of.openTag("vehicle").writeAttr("id", veh->getID()).writeAttr("eclass", PollutantsInterface::getName(veh->getVehicleType().getEmissionClass())).writeAttr("CO2", veh->getCO2Emissions());
-        of.writeAttr("CO", veh->getCOEmissions()).writeAttr("HC", veh->getHCEmissions()).writeAttr("NOx", veh->getNOxEmissions());
-        of.writeAttr("PMx", veh->getPMxEmissions()).writeAttr("fuel", veh->getFuelConsumption()).writeAttr("noise", veh->getHarmonoise_NoiseEmissions());
-        of.writeAttr("route", veh->getRoute().getID()).writeAttr("type", fclass).writeAttr("waiting", veh->getWaitingSeconds());
-        of.writeAttr("lane", veh->getLane()->getID()).writeAttr("pos", veh->getPositionOnLane()).writeAttr("speed", veh->getSpeed());
-        of.writeAttr("angle", veh->getAngle()).writeAttr("x", pos.x()).writeAttr("y", pos.y());
-        of.closeTag();
     }
     of.closeTag();
 }
