@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 25 July 2005
-/// @version $Id: MSTriggeredRerouter.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSTriggeredRerouter.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Reroutes vehicles passing an edge
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -76,7 +76,7 @@ public:
      * @param[in] file The file to read the reroute definitions from
      */
     MSTriggeredRerouter(const std::string& id,
-                        const std::vector<MSEdge*>& edges,
+                        const MSEdgeVector& edges,
                         SUMOReal prob, const std::string& file, bool off);
 
 
@@ -94,11 +94,15 @@ public:
         /// The end time these definitions are valid
         SUMOTime end;
         /// The list of closed edges
-        std::vector<MSEdge*> closed;
+        MSEdgeVector closed;
         /// The distributions of new destinations to use
         RandomDistributor<MSEdge*> edgeProbs;
         /// The distributions of new routes to use
         RandomDistributor<const MSRoute*> routeProbs;
+        /// The permissions to use
+        SVCPermissions permissions;
+        /// The old permissions for all lanes
+        std::vector<SVCPermissions> prevPermissions;
     };
 
     /** @brief Tries to reroute the vehicle
@@ -117,17 +121,14 @@ public:
      */
     bool notifyEnter(SUMOVehicle& veh, MSMoveReminder::Notification reason);
 
-    /// Returns whether a rerouting definition is valid for the given time and vehicle
-    bool hasCurrentReroute(SUMOTime time, SUMOVehicle& veh) const;
+    /// Returns the rerouting definition valid for the given time and vehicle, 0 if none
+    const RerouteInterval* getCurrentReroute(SUMOTime time, SUMOVehicle& veh) const;
 
-    /// Returns the rerouting definition valid for the given time and vehicle
-    const RerouteInterval& getCurrentReroute(SUMOTime time, SUMOVehicle& veh) const;
+    /// Sets the edge permission if there are any defined in the closingEdge
+    SUMOTime setPermissions(const SUMOTime currentTime);
 
-    /// Returns whether a rerouting definition is valid for the given time
-    bool hasCurrentReroute(SUMOTime time) const;
-
-    /// Returns the rerouting definition valid for the given time and vehicle
-    const RerouteInterval& getCurrentReroute(SUMOTime time) const;
+    /// Returns the rerouting definition valid for the given time, 0 if none
+    const RerouteInterval* getCurrentReroute(SUMOTime time) const;
 
     /// Sets whether the process is currently steered by the user
     void setUserMode(bool val);
@@ -165,7 +166,7 @@ protected:
      * @exception ProcessError If something fails
      * @see GenericSAXHandler::myEndElement
      */
-    void myEndElement(int element);
+    virtual void myEndElement(int element);
     //@}
 
 protected:
@@ -184,7 +185,9 @@ protected:
     /// The first and the last time steps of the interval
     SUMOTime myCurrentIntervalBegin, myCurrentIntervalEnd;
     /// List of closed edges
-    std::vector<MSEdge*> myCurrentClosed;
+    MSEdgeVector myCurrentClosed;
+    /// List of permissions for closed edges
+    SVCPermissions myCurrentPermissions;
     /// new destinations with probabilities
     RandomDistributor<MSEdge*> myCurrentEdgeProb;
     /// new routes with probabilities
