@@ -3,12 +3,12 @@
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    02. March 2012
-/// @version $Id: NBAlgorithms.cpp 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NBAlgorithms.cpp 18120 2015-03-20 13:44:18Z namdre $
 ///
 // Algorithms for network computation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -68,22 +68,16 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node) {
         NBEdge* outedge = *j;
         for (std::vector<NBEdge*>::const_iterator k = incoming.begin(); k != incoming.end(); ++k) {
             NBEdge* e = *k;
-            if (e->getConnections().size() != 0 && !e->isConnectedTo(outedge)) {
-                // has connections, but not to outedge; outedge will not be the turn direction
-                //
-                // @todo: this seems to be needed due to legacy issues; actually, we could regard
-                //  such pairs, too, and it probably would increase the accuracy. But there is
-                //  no mechanism implemented, yet, which would avoid adding them as turnarounds though
-                //  no connection is specified.
-                continue;
-            }
-
             // @todo: check whether NBHelpers::relAngle is properly defined and whether it should really be used, here
-            SUMOReal angle = fabs(NBHelpers::relAngle(e->getAngleAtNode(node), outedge->getAngleAtNode(node)));
-            if (angle < 160) {
+            const SUMOReal signedAngle = NBHelpers::normRelAngle(e->getAngleAtNode(node), outedge->getAngleAtNode(node));
+            if (signedAngle > 0 && signedAngle < 177 && e->getGeometry().back().distanceTo2D(outedge->getGeometry().front()) < POSITION_EPS) {
+                // backwards curving edges can only be turnaround when there are
+                // non-default endpoints
                 continue;
             }
-            if (e->getFromNode() == outedge->getToNode()) {
+            SUMOReal angle = fabs(signedAngle);
+            // std::cout << "incoming=" << e->getID() << " outgoing=" << outedge->getID() << " relAngle=" << NBHelpers::relAngle(e->getAngleAtNode(node), outedge->getAngleAtNode(node)) << "\n";
+            if (e->getFromNode() == outedge->getToNode() && angle > 120) {
                 // they connect the same nodes; should be the turnaround direction
                 // we'll assign a maximum number
                 //
@@ -96,6 +90,9 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node) {
                 //  be realistic in any means; we will warn, here.
                 angle += 360;
             }
+            if (angle < 160) {
+                continue;
+            }
             Combination c;
             c.from = e;
             c.to = outedge;
@@ -120,7 +117,9 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node) {
         seen.insert((*j).from);
         seen.insert((*j).to);
         // set turnaround information
-        (*j).from->setTurningDestination((*j).to);
+        bool onlyPossible = (*j).from->getConnections().size() != 0 && !(*j).from->isConnectedTo((*j).to);
+        //std::cout << "    setTurningDestination from=" << (*j).from->getID() << " to=" << (*j).to->getID() << " onlyPossible=" << onlyPossible << "\n";
+        (*j).from->setTurningDestination((*j).to, onlyPossible);
     }
 }
 
@@ -129,7 +128,7 @@ NBTurningDirectionsComputer::computeTurnDirectionsForNode(NBNode* node) {
 // NBNodesEdgesSorter
 // ---------------------------------------------------------------------------
 void
-NBNodesEdgesSorter::sortNodesEdges(NBNodeCont& nc, bool leftHand) {
+NBNodesEdgesSorter::sortNodesEdges(NBNodeCont& nc, bool leftHand, bool useNodeShape) {
     for (std::map<std::string, NBNode*>::const_iterator i = nc.begin(); i != nc.end(); ++i) {
         NBNode* n = (*i).second;
         if (n->myAllEdges.size() == 0) {
@@ -139,19 +138,41 @@ NBNodesEdgesSorter::sortNodesEdges(NBNodeCont& nc, bool leftHand) {
         EdgeVector& incoming = (*i).second->myIncomingEdges;
         EdgeVector& outgoing = (*i).second->myOutgoingEdges;
         std::vector<NBNode::Crossing>& crossings = (*i).second->myCrossings;
-        // sort the edges
-        std::sort(allEdges.begin(), allEdges.end(), edge_by_junction_angle_sorter(n));
-        std::sort(incoming.begin(), incoming.end(), edge_by_junction_angle_sorter(n));
-        std::sort(outgoing.begin(), outgoing.end(), edge_by_junction_angle_sorter(n));
-        std::vector<NBEdge*>::iterator j;
-        for (j = allEdges.begin(); j != allEdges.end() - 1 && j != allEdges.end(); ++j) {
-            swapWhenReversed(n, leftHand, j, j + 1);
-        }
-        if (allEdges.size() > 1 && j != allEdges.end()) {
-            swapWhenReversed(n, leftHand, allEdges.end() - 1, allEdges.begin());
+
+        if (!useNodeShape || n->getShape().area() < 1) {
+            // if the area is to small (i.e. for simple-continuation nodes) we better not use it
+            // sort by the angle of the adjoining line segment of the edge geometry
+            // sort the edges
+            std::sort(allEdges.begin(), allEdges.end(), edge_by_junction_angle_sorter(n));
+            std::sort(incoming.begin(), incoming.end(), edge_by_junction_angle_sorter(n));
+            std::sort(outgoing.begin(), outgoing.end(), edge_by_junction_angle_sorter(n));
+            std::vector<NBEdge*>::iterator j;
+            for (j = allEdges.begin(); j != allEdges.end() - 1 && j != allEdges.end(); ++j) {
+                swapWhenReversed(n, leftHand, j, j + 1);
+            }
+            if (allEdges.size() > 1 && j != allEdges.end()) {
+                swapWhenReversed(n, leftHand, allEdges.end() - 1, allEdges.begin());
+            }
+        } else {
+            NBEdge* firstOfAll = allEdges.front();
+            NBEdge* firstOfIncoming = incoming.size() > 0 ? incoming.front() : 0;
+            NBEdge* firstOfOutgoing = outgoing.size() > 0 ? outgoing.front() : 0;
+            // sort by the angle between the node shape center and the point where the edge meeds the node shape
+            sort(allEdges.begin(), allEdges.end(), NBContHelper::edge_by_angle_to_nodeShapeCentroid_sorter(n));
+            sort(incoming.begin(), incoming.end(), NBContHelper::edge_by_angle_to_nodeShapeCentroid_sorter(n));
+            sort(outgoing.begin(), outgoing.end(), NBContHelper::edge_by_angle_to_nodeShapeCentroid_sorter(n));
+            // let the first edge remain the first
+            rotate(allEdges.begin(), std::find(allEdges.begin(), allEdges.end(), firstOfAll), allEdges.end());
+            if (firstOfIncoming != 0) {
+                rotate(incoming.begin(), std::find(incoming.begin(), incoming.end(), firstOfIncoming), incoming.end());
+            }
+            if (firstOfOutgoing != 0) {
+                rotate(outgoing.begin(), std::find(outgoing.begin(), outgoing.end(), firstOfOutgoing), outgoing.end());
+            }
         }
+
         // sort the crossings
-        std::sort(crossings.begin(), crossings.end(), crossing_by_junction_angle_sorter(allEdges));
+        std::sort(crossings.begin(), crossings.end(), crossing_by_junction_angle_sorter(n, allEdges));
         // DEBUG
         //if (n->getID() == "cluster_492462300_671564296") {
         //    if (crossings.size() > 0) {
@@ -179,7 +200,7 @@ NBNodesEdgesSorter::swapWhenReversed(const NBNode* const n, bool leftHand,
     //  is not nice. Maybe we could get rid of it if we would always mark edges
     //  as turnarounds, even if they do not have to be added, as mentioned in
     //  notes on NBTurningDirectionsComputer::computeTurnDirectionsForNode
-    if (e2->getToNode() == n && e2->isTurningDirectionAt(n, e1)) {
+    if (e2->getToNode() == n && e2->isTurningDirectionAt(e1)) {
         std::swap(*i1, *i2);
     }
 }
@@ -287,7 +308,7 @@ NBEdgePriorityComputer::setPriorityJunctionPriorities(NBNode& n) {
                                            && (incoming.size() == 0 || bestIncoming[0]->getPriority() > incoming[0]->getPriority())
                                            && bestOutgoing.size() == 1 && n.myOutgoingEdges.size() <= 2
                                            && (outgoing.size() == 0 || bestOutgoing[0]->getPriority() > outgoing[0]->getPriority())
-                                           && !bestIncoming[0]->isTurningDirectionAt(&n, bestOutgoing[0]));
+                                           && !bestIncoming[0]->isTurningDirectionAt(bestOutgoing[0]));
     // now, let's compute for each of the best incoming edges
     //  the incoming which is most opposite
     //  the outgoing which is most opposite
@@ -341,13 +362,13 @@ NBEdgePriorityComputer::setPriorityJunctionPriorities(NBNode& n) {
     for (i = bestIncoming.begin(); i != bestIncoming.end(); ++i) {
         EdgeVector::iterator j;
         NBEdge* t1 = *i;
-        SUMOReal angle1 = t1->getTotalAngle() + 180;
+        SUMOReal angle1 = t1->getAngleAtNode(&n) + 180;
         if (angle1 >= 360) {
             angle1 -= 360;
         }
         for (j = i + 1; j != bestIncoming.end(); ++j) {
             NBEdge* t2 = *j;
-            SUMOReal angle2 = t2->getTotalAngle() + 180;
+            SUMOReal angle2 = t2->getAngleAtNode(&n) + 180;
             if (angle2 >= 360) {
                 angle2 -= 360;
             }
@@ -400,5 +421,14 @@ NBEdgePriorityComputer::samePriority(const NBEdge* const e1, const NBEdge* const
 }
 
 
+NBNodesEdgesSorter::crossing_by_junction_angle_sorter::crossing_by_junction_angle_sorter(const NBNode* node, const EdgeVector& ordering) {
+    // reorder based on getAngleAtNodeToCenter
+    myOrdering = ordering;
+    sort(myOrdering.begin(), myOrdering.end(), NBContHelper::edge_by_angle_to_nodeShapeCentroid_sorter(node));
+    // let the first edge remain the first
+    rotate(myOrdering.begin(), std::find(myOrdering.begin(), myOrdering.end(), ordering.front()), myOrdering.end());
+}
+
+
 /****************************************************************************/
 
