@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdgeCont.cpp 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NBEdgeCont.cpp 18172 2015-03-27 17:02:39Z behrisch $
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -86,6 +86,18 @@ NBEdgeCont::applyOptions(OptionsCont& oc) {
     // set edges dismiss/accept options
     myEdgesMinSpeed = oc.isSet("keep-edges.min-speed") ? oc.getFloat("keep-edges.min-speed") : -1;
     myRemoveEdgesAfterJoining = oc.exists("keep-edges.postload") && oc.getBool("keep-edges.postload");
+    // we possibly have to load the edges to keep
+    if (oc.isSet("keep-edges.input-file")) {
+        std::ifstream strm(oc.getString("keep-edges.input-file").c_str());
+        if (!strm.good()) {
+            throw ProcessError("Could not load names of edges too keep from '" + oc.getString("keep-edges.input-file") + "'.");
+        }
+        while (strm.good()) {
+            std::string name;
+            strm >> name;
+            myEdges2Keep.insert(name);
+        }
+    }
     if (oc.isSet("keep-edges.explicit")) {
         const std::vector<std::string> edges = oc.getStringVector("keep-edges.explicit");
         myEdges2Keep.insert(edges.begin(), edges.end());
@@ -229,6 +241,7 @@ NBEdgeCont::ignoreFilterMatch(NBEdge* edge) {
             } else {
                 WRITE_ERROR("Cannot prune edges using a geo-boundary because no projection has been loaded");
             }
+            myNeedGeoTransformedPrunningBoundary = false;
         }
         if (!(edge->getGeometry().getBoxBoundary().grow((SUMOReal) POSITION_EPS).overlapsWith(myPrunningBoundary))) {
             return true;
@@ -399,7 +412,8 @@ NBEdgeCont::splitAt(NBDistrictCont& dc, NBEdge* edge, NBNode* node,
                     const std::string& firstEdgeName,
                     const std::string& secondEdgeName,
                     unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge,
-                    const SUMOReal speed) {
+                    const SUMOReal speed,
+                    const int changedLeft) {
     SUMOReal pos;
     pos = edge->getGeometry().nearest_offset_to_point2D(node->getPosition());
     if (pos <= 0) {
@@ -411,7 +425,7 @@ NBEdgeCont::splitAt(NBDistrictCont& dc, NBEdge* edge, NBNode* node,
         return false;
     }
     return splitAt(dc, edge, pos, node, firstEdgeName, secondEdgeName,
-                   noLanesFirstEdge, noLanesSecondEdge, speed);
+                   noLanesFirstEdge, noLanesSecondEdge, speed, changedLeft);
 }
 
 
@@ -421,7 +435,13 @@ NBEdgeCont::splitAt(NBDistrictCont& dc,
                     const std::string& firstEdgeName,
                     const std::string& secondEdgeName,
                     unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge,
-                    const SUMOReal speed) {
+                    const SUMOReal speed,
+                    const int changedLeft
+                   ) {
+    // there must be at least some overlap between first and second edge
+    assert(changedLeft > -((int)noLanesFirstEdge));
+    assert(changedLeft < (int)noLanesSecondEdge);
+
     // build the new edges' geometries
     std::pair<PositionVector, PositionVector> geoms =
         edge->getGeometry().splitAt(pos);
@@ -449,20 +469,14 @@ NBEdgeCont::splitAt(NBDistrictCont& dc,
     edge->myFrom->removeDoubleEdges();
     edge->myTo->removeDoubleEdges();
     // add connections from the first to the second edge
-    // check special case:
-    //  one in, one out, the outgoing has one lane more
-    if (noLanesFirstEdge == noLanesSecondEdge - 1) {
-        for (unsigned int i = 0; i < one->getNumLanes(); i++) {
-            if (!one->addLane2LaneConnection(i, two, i + 1, NBEdge::L2L_COMPUTED)) { // !!! Bresenham, here!!!
-                throw ProcessError("Could not set connection!");
-            }
-        }
-        one->addLane2LaneConnection(0, two, 0, NBEdge::L2L_COMPUTED);
-    } else {
-        for (unsigned int i = 0; i < one->getNumLanes() && i < two->getNumLanes(); i++) {
-            if (!one->addLane2LaneConnection(i, two, i, NBEdge::L2L_COMPUTED)) {// !!! Bresenham, here!!!
-                throw ProcessError("Could not set connection!");
-            }
+    // there will be as many connections as there are lanes on the second edge
+    // by default lanes will be added / discontinued on the right side
+    // (appropriate for highway on-/off-ramps)
+    const int offset = (int)one->getNumLanes() - (int)two->getNumLanes() + changedLeft;
+    for (int i2 = 0; i2 < (int)two->getNumLanes(); i2++) {
+        const int i1 = MIN2(MAX2((int)0, i2 + offset), (int)one->getNumLanes());
+        if (!one->addLane2LaneConnection(i1, two, i2, NBEdge::L2L_COMPUTED)) {
+            throw ProcessError("Could not set connection!");
         }
     }
     if (myRemoveEdgesAfterJoining) {
@@ -654,11 +668,20 @@ NBEdgeCont::joinSameNodeConnectingEdges(NBDistrictCont& dc,
     }
     speed /= edges.size();
     // build the new edge
-    // @bug new edge does not know about allowed vclass of old edges
-    // @bug both the width and the offset are not regarded
     NBEdge* newEdge = new NBEdge(id, from, to, "", speed, nolanes, priority,
                                  NBEdge::UNSPECIFIED_WIDTH, NBEdge::UNSPECIFIED_OFFSET,
                                  tpledge->getStreetName(), tpledge->myLaneSpreadFunction);
+    // copy lane attributes
+    int laneIndex = 0;
+    for (i = edges.begin(); i != edges.end(); ++i) {
+        const std::vector<NBEdge::Lane>& lanes = (*i)->getLanes();
+        for (int j = 0; j < (int)lanes.size(); ++j) {
+            newEdge->setPermissions(lanes[j].permissions, laneIndex);
+            newEdge->setLaneWidth(laneIndex, lanes[j].width);
+            newEdge->setEndOffset(laneIndex, lanes[j].endOffset);
+            laneIndex++;
+        }
+    }
     insert(newEdge, true);
     // replace old edge by current within the nodes
     //  and delete the old
@@ -968,11 +991,17 @@ NBEdgeCont::generateStreetSigns() {
 
 
 int
-NBEdgeCont::guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed) {
+NBEdgeCont::guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed, bool fromPermissions) {
     int sidewalksCreated = 0;
     for (EdgeCont::iterator it = myEdges.begin(); it != myEdges.end(); it++) {
         NBEdge* edge = it->second;
-        if (edge->getSpeed() > minSpeed && edge->getSpeed() <= maxSpeed && edge->getPermissions(0) != SVC_PEDESTRIAN) {
+        if ((
+                    // guess.from-permissions
+                    (fromPermissions && (edge->getPermissions() & SVC_PEDESTRIAN) != 0)
+                    // guess from speed
+                    || (!fromPermissions && edge->getSpeed() > minSpeed && edge->getSpeed() <= maxSpeed))
+                // does not yet have a sidewalk
+                && edge->getPermissions(0) != SVC_PEDESTRIAN) {
             edge->addSidewalk(width);
             sidewalksCreated += 1;
         }
