@@ -3,13 +3,13 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mar 2011
-/// @version $Id: NBLoadedSUMOTLDef.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBLoadedSUMOTLDef.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // A complete traffic light logic loaded from a sumo-net. (opted to reimplement
 // since NBLoadedTLDef is quite vissim specific)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2011-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2011-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -126,12 +126,20 @@ public:
      */
     void setOffset(SUMOTime offset);
 
+    /** @brief Sets the algorithm type of this tls
+     * @param[in] offset The algorithm type of this tls
+     */
+    void setType(TrafficLightType type);
+
 protected:
     /** @brief Collects the links participating in this traffic light
      *    (only if not previously loaded)
      */
     void collectLinks();
 
+    /** @brief Build the list of participating edges
+     */
+    void collectEdges();
 
     /** @brief Computes the traffic light logic finally in dependence to the type
      * @param[in] ec The edge container
