@@ -8,12 +8,12 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBNodeCont.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBNodeCont.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Container for nodes during the netbuilding process
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -371,6 +371,13 @@ NBNodeCont::removeUnwishedNodes(NBDistrictCont& dc, NBEdgeCont& ec,
 }
 
 
+void
+NBNodeCont::avoidOverlap() {
+    for (NodeCont::iterator i = myNodes.begin(); i != myNodes.end(); i++) {
+        (*i).second->avoidOverlap();
+    }
+}
+
 // ----------- (Helper) methods for joining nodes
 void
 NBNodeCont::generateNodeClusters(SUMOReal maxDist, NodeClusters& into) const {
@@ -1051,6 +1058,7 @@ NBNodeCont::printBuiltNodesStatistics() const {
     int numPriorityJunctions = 0;
     int numRightBeforeLeftJunctions = 0;
     int numAllWayStopJunctions = 0;
+    int numRailSignals = 0;
     for (NodeCont::const_iterator i = myNodes.begin(); i != myNodes.end(); i++) {
         switch ((*i).second->getType()) {
             case NODETYPE_NOJUNCTION:
@@ -1076,6 +1084,9 @@ NBNodeCont::printBuiltNodesStatistics() const {
                 break;
             case NODETYPE_UNKNOWN:
                 break;
+            case NODETYPE_RAIL_SIGNAL:
+                ++numRailSignals;
+                break;
             default:
                 break;
         }
@@ -1090,6 +1101,9 @@ NBNodeCont::printBuiltNodesStatistics() const {
     if (numAllWayStopJunctions > 0) {
         WRITE_MESSAGE("  All-way stop junctions      : " + toString(numAllWayStopJunctions));
     }
+    if (numRailSignals > 0) {
+        WRITE_MESSAGE("  Rail signal junctions      : " + toString(numRailSignals));
+    }
 }
 
 
