@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 04.05.2011
-/// @version $Id: NWWriter_SUMO.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NWWriter_SUMO.h 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // Exporter writing networks using the SUMO format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -93,9 +93,6 @@ public:
     /// @brief writes the traffic light logics to the given device
     static void writeTrafficLights(OutputDevice& into, const NBTrafficLightLogicCont& tllCont);
 
-    /// @brief writes the location element
-    static void writeLocation(OutputDevice& into);
-
     /// @brief writes allowed disallowed attributes if needed;
     static void writePermissions(OutputDevice& into, SVCPermissions permissions);
 
@@ -140,11 +137,13 @@ protected:
      * @param[in] length Lane's length
      * @param[in] index The index of the lane within the edge
      * @param[in] origNames Whether original names shall be written as parameter
+     * @param[in] node The node to check for custom shape data
      */
     static void writeLane(OutputDevice& into, const std::string& eID, const std::string& lID,
                           SUMOReal speed, SVCPermissions permissions, SVCPermissions preferred,
-                          SUMOReal endOffset, SUMOReal width, const PositionVector& shape,
-                          const std::string& origID, SUMOReal length, unsigned int index, bool origNames);
+                          SUMOReal endOffset, SUMOReal width, PositionVector shape,
+                          const std::string& origID, SUMOReal length, unsigned int index, bool origNames,
+                          const NBNode* node = 0);
 
 
     /** @brief Writes a junction (<junction ...)
