@@ -6,12 +6,12 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: polyconvert_main.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: polyconvert_main.cpp 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // Main for POLYCONVERT
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2005-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -203,6 +203,10 @@ fillOptions() {
 
     oc.doRegister("discard", new Option_Bool(false));
     oc.addDescription("discard", "Building Defaults", "Sets default action to discard");
+
+    // projection
+    oc.doRegister("proj.plain-geo", new Option_Bool(false));
+    oc.addDescription("proj.plain-geo", "Projection", "Write geo coordinates in output");
 }
 
 
@@ -287,13 +291,14 @@ main(int argc, char** argv) {
         PCLoaderDlrNavteq::loadIfSet(oc, toFill, tm); // Elmar-files
         PCLoaderVisum::loadIfSet(oc, toFill, tm); // VISUM
         PCLoaderArcView::loadIfSet(oc, toFill, tm); // shape-files
+        GeoConvHelper::computeFinal();
         // error processing
         if (MsgHandler::getErrorInstance()->wasInformed() && oc.getBool("ignore-errors")) {
             MsgHandler::getErrorInstance()->clear();
         }
         if (!MsgHandler::getErrorInstance()->wasInformed()) {
             // no? ok, save
-            toFill.save(oc.getString("output-file"));
+            toFill.save(oc.getString("output-file"), oc.getBool("proj.plain-geo"));
         } else {
             throw ProcessError();
         }
