@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONetHandler.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: RONetHandler.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // The handler that parses a SUMO-network for its usage in a router
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -147,6 +147,13 @@ protected:
     void parseBusStop(const SUMOSAXAttributes& attrs);
 
 
+    /** @begin Parses a container stop
+     * Called on the occurence of a "containerStop" element
+     * @param[in] attrs The attributes (of the "containerStop"-element) to parse
+     */
+    void parseContainerStop(const SUMOSAXAttributes& attrs);
+
+
     /** @begin Parses a district and creates a pseudo edge for it
      *
      * Called on the occurence of a "district" element, this method
