@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Walter Bamberger
 /// @date    2006-01-24
-/// @version $Id: SUMOVehicleClass.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: SUMOVehicleClass.h 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // Definitions of SUMO vehicle classes and helper functions
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -93,7 +93,9 @@ enum SUMOVehicleShape {
     /// @brief render as a (futuristic) e-vehicle
     SVS_E_VEHICLE,
     /// @brief render as a giant ant
-    SVS_ANT
+    SVS_ANT,
+    /// @brief render as a arbitrary ship
+    SVS_SHIP
 };
 
 
@@ -176,10 +178,12 @@ enum SUMOVehicleClass {
     SVC_PEDESTRIAN = 1 << 20,
     /// @brief is an electric vehicle
     SVC_E_VEHICLE = 1 << 21,
+    /// @brief is an arbitrary ship
+    SVC_SHIP = 1 << 22,
     /// @brief is a user-defined type
-    SVC_CUSTOM1 = 1 << 22,
+    SVC_CUSTOM1 = 1 << 23,
     /// @brief is a user-defined type
-    SVC_CUSTOM2 = 1 << 23
+    SVC_CUSTOM2 = 1 << 24
                   //@}
 };
 
@@ -191,7 +195,8 @@ extern StringBijection<SUMOVehicleShape> SumoVehicleShapeStrings;
 /* @brief bitset where each bit declares whether a certain SVC may use this edge/lane
  */
 typedef int SVCPermissions;
-extern const SVCPermissions SVCAll;
+extern const SVCPermissions SVCAll; // everything allowed
+extern const SVCPermissions SVC_UNSPECIFIED; // permissions not specified
 
 
 /**
@@ -295,6 +300,12 @@ extern SUMOVehicleShape getVehicleShapeID(const std::string& name);
  */
 extern bool isRailway(SVCPermissions permissions);
 
+/** @brief Returns whether an edge with the given permission is a waterway edge
+ * @param[in] permissions The permissions of the edge
+ * @return Whether the edge is a waterway edge
+ */
+extern bool isWaterway(SVCPermissions permissions);
+
 /** @brief Returns whether an edge with the given permission is a forbidden edge
  * @param[in] permissions The permissions of the edge
  * @return Whether the edge is forbidden
@@ -305,11 +316,14 @@ extern bool isForbidden(SVCPermissions permissions);
 // default vehicle type parameter
 // ---------------------------------------------------------------------------
 extern const std::string DEFAULT_VTYPE_ID;
+extern const std::string DEFAULT_PEDTYPE_ID;
 
 extern const SUMOReal DEFAULT_VEH_PROB; // !!! does this belong here?
 
 extern const SUMOReal DEFAULT_PEDESTRIAN_SPEED;
 
+extern const SUMOReal DEFAULT_CONTAINER_TRANSHIP_SPEED;
+
 #endif
 
 /****************************************************************************/
