@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: GUIParameterTableWindow.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIParameterTableWindow.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // The window that holds the table of an object's parameter
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -60,6 +60,12 @@ FXDEFMAP(GUIParameterTableWindow) GUIParameterTableWindowMap[] = {
 FXIMPLEMENT(GUIParameterTableWindow, FXMainWindow, GUIParameterTableWindowMap, ARRAYNUMBER(GUIParameterTableWindowMap))
 
 
+// ===========================================================================
+// static value definitions
+// ===========================================================================
+MFXMutex GUIParameterTableWindow::myGlobalContainerLock;
+std::vector<GUIParameterTableWindow*> GUIParameterTableWindow::myContainer;
+
 // ===========================================================================
 // method definitions
 // ===========================================================================
@@ -89,6 +95,8 @@ GUIParameterTableWindow::GUIParameterTableWindow(GUIMainWindow& app,
     myLock.lock();
     myObject->addParameterTable(this);
     myLock.unlock();
+    AbstractMutex::ScopedLocker locker(myGlobalContainerLock);
+    myContainer.push_back(this);
 }
 
 
@@ -102,6 +110,11 @@ GUIParameterTableWindow::~GUIParameterTableWindow() {
         myObject->removeParameterTable(this);
     }
     myLock.unlock();
+    AbstractMutex::ScopedLocker locker(myGlobalContainerLock);
+    std::vector<GUIParameterTableWindow*>::iterator i = std::find(myContainer.begin(), myContainer.end(), this);
+    if (i != myContainer.end()) {
+        myContainer.erase(i);
+    }
 }
 
 
@@ -114,6 +127,7 @@ GUIParameterTableWindow::removeObject(GUIGlObject* /*i*/) {
 
 long
 GUIParameterTableWindow::onSimStep(FXObject*, FXSelector, void*) {
+    // table values are updated in GUINet::guiSimulationStep()
     updateTable();
     update();
     return 1;
