@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: GUIParameterTableWindow.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIParameterTableWindow.h 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // The window that holds the table of an object's parameter
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -34,6 +34,8 @@
 
 #include <vector>
 #include <string>
+#include <algorithm>
+#include <functional>
 #include <fx.h>
 #include <utils/foxtools/MFXMutex.h>
 #include <utils/common/ValueSource.h>
@@ -201,6 +203,12 @@ public:
     long onRightButtonPress(FXObject*, FXSelector, void*);
     /// @}
 
+    /** @brief Updates all instances
+     */
+    static void updateAll() {
+        AbstractMutex::ScopedLocker locker(myGlobalContainerLock);
+        std::for_each(myContainer.begin(), myContainer.end(), std::mem_fun(&GUIParameterTableWindow::updateTable));
+    }
 
 protected:
     /** @brief Updates the table
@@ -211,6 +219,11 @@ protected:
      */
     void updateTable();
 
+    /// @brief The mutex used to avoid concurrent updates of the instance container
+    static MFXMutex myGlobalContainerLock;
+
+    /// @brief The container of items that shall be updated
+    static std::vector<GUIParameterTableWindow*> myContainer;
 
 private:
     /// @brief The object to get the information from
@@ -228,7 +241,7 @@ private:
     /// @brief The index of the next row to add - used while building
     unsigned myCurrentPos;
 
-    /// @brief A lock assuring save updates in cse of object deletion
+    /// @brief A lock assuring save updates in case of object deletion
     mutable MFXMutex myLock;
 
 protected:
