@@ -6,12 +6,12 @@
 /// @author  Laura Bieker
 /// @author  Andreas Gaubatz
 /// @date    Sept 2002
-/// @version $Id: GUISUMOAbstractView.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUISUMOAbstractView.cpp 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // The base class for a view
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -508,7 +508,9 @@ GUISUMOAbstractView::centerTo(GUIGlID id, bool applyZoom, SUMOReal zoomDist) {
     if (o != 0 && dynamic_cast<GUIGlObject*>(o) != 0) {
         if (applyZoom && zoomDist < 0) {
             myChanger->setViewport(o->getCenteringBoundary());
+            update(); // only update when centering onto an object once
         } else {
+            // called during tracking. update is triggered somewhere else
             myChanger->centerTo(o->getCenteringBoundary().getCenter(), zoomDist, applyZoom);
         }
     }
@@ -655,11 +657,7 @@ GUISUMOAbstractView::onMouseMove(FXObject*, FXSelector , void* data) {
     if (myViewportChooser == 0 || !myViewportChooser->haveGrabbed()) {
         myChanger->onMouseMove(data);
     }
-    const SUMOReal xpos = myChanger->getXPos();
-    const SUMOReal ypos = myChanger->getYPos();
-    const SUMOReal zoom = myChanger->getZoom();
-    if (myViewportChooser != 0 &&
-            (xpos != myChanger->getXPos() || ypos != myChanger->getYPos() || zoom != myChanger->getZoom())) {
+    if (myViewportChooser != 0) {
         myViewportChooser->setValues(myChanger->getZoom(), myChanger->getXPos(), myChanger->getYPos());
     }
     updatePositionInformation();
@@ -921,14 +919,21 @@ GUISUMOAbstractView::showViewschemeEditor() {
     myVisualizationChanger->show();
 }
 
-
-void
-GUISUMOAbstractView::showViewportEditor() {
+GUIDialog_EditViewport*
+GUISUMOAbstractView::getViewportEditor() {
     if (myViewportChooser == 0) {
         myViewportChooser =
             new GUIDialog_EditViewport(this, "Edit Viewport...", 0, 0);
         myViewportChooser->create();
     }
+    myViewportChooser->setValues(myChanger->getZoom(), myChanger->getXPos(), myChanger->getYPos());
+    return myViewportChooser;
+}
+
+
+void
+GUISUMOAbstractView::showViewportEditor() {
+    getViewportEditor(); // make sure it exists;
     Position p(myChanger->getXPos(), myChanger->getYPos(), myChanger->getZoom());
     myViewportChooser->setOldValues(p, Position::INVALID);
     myViewportChooser->show();
@@ -1135,5 +1140,11 @@ GUISUMOAbstractView::applyGLTransform(bool fixRatio) {
     glTranslated(-bound.xmin(), -bound.ymin(), 0);
 }
 
+
+SUMOReal
+GUISUMOAbstractView::getDelay() const {
+    return myApp->getDelay();
+}
+
 /****************************************************************************/
 
