@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    2004
-/// @version $Id: OutputDevice.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: OutputDevice.cpp 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // Static storage of an output device and its base (abstract) implementation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2004-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -86,7 +86,20 @@ OutputDevice::getDevice(const std::string& name) {
         }
     } else {
         const size_t len = name.length();
-        dev = new OutputDevice_File(name, len > 4 && name.substr(len - 4) == ".sbx");
+        std::string name2 = name;
+        if (OptionsCont::getOptions().isSet("output-prefix")) {
+            std::string prefix = OptionsCont::getOptions().getString("output-prefix");
+            size_t metaTimeIndex = prefix.find("TIME");
+            if (metaTimeIndex != std::string::npos) {
+                time_t rawtime;
+                char buffer [80];
+                time(&rawtime);
+                strftime(buffer, 80, "%F-%H-%M-%S", localtime(&rawtime));
+                prefix.replace(metaTimeIndex, 4, std::string(buffer));
+            }
+            name2 = FileHelpers::prependToLastPathComponent(prefix, name);
+        }
+        dev = new OutputDevice_File(name2, len > 4 && name.substr(len - 4) == ".sbx");
     }
     dev->setPrecision();
     dev->getOStream() << std::setiosflags(std::ios::fixed);
