@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Jun 2004
-/// @version $Id: Polygon.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: Polygon.cpp 18137 2015-03-24 15:12:38Z behrisch $
 ///
 // A 2D-polygon
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2004-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2004-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -30,6 +30,9 @@
 #include <config.h>
 #endif
 
+#include <utils/iodevices/OutputDevice.h>
+#include <utils/common/StringUtils.h>
+#include <utils/geom/GeoConvHelper.h>
 #include "Polygon.h"
 using namespace SUMO;
 
@@ -53,6 +56,36 @@ Polygon::Polygon(const std::string& id, const std::string& type,
 Polygon::~Polygon() {}
 
 
+void
+Polygon::writeXML(OutputDevice& out, bool geo) {
+    out.openTag(SUMO_TAG_POLY);
+    out.writeAttr(SUMO_ATTR_ID, StringUtils::escapeXML(getID()));
+    out.writeAttr(SUMO_ATTR_TYPE, StringUtils::escapeXML(getType()));
+    out.writeAttr(SUMO_ATTR_COLOR, getColor());
+    out.writeAttr(SUMO_ATTR_FILL,  getFill());
+    out.writeAttr(SUMO_ATTR_LAYER, getLayer());
+    PositionVector shape = getShape();
+    if (geo) {
+        out.writeAttr(SUMO_ATTR_GEO, true);
+        for (int i = 0; i < (int) shape.size(); i++) {
+            GeoConvHelper::getFinal().cartesian2geo(shape[i]);
+        }
+    }
+    out.writeAttr(SUMO_ATTR_SHAPE, shape);
+    if (getAngle() != Shape::DEFAULT_ANGLE) {
+        out.writeAttr(SUMO_ATTR_ANGLE, getAngle());
+    }
+    if (getImgFile() != Shape::DEFAULT_IMG_FILE) {
+        out.writeAttr(SUMO_ATTR_IMGFILE, getImgFile());
+    }
+    for (std::map<std::string, std::string>::const_iterator j = getMap().begin(); j != getMap().end(); ++j) {
+        out.openTag(SUMO_TAG_PARAM);
+        out.writeAttr(SUMO_ATTR_KEY, (*j).first);
+        out.writeAttr(SUMO_ATTR_VALUE, (*j).second);
+        out.closeTag();
+    }
+    out.closeTag();
+}
 
 /****************************************************************************/
 
