@@ -2,12 +2,12 @@
 /// @file    PedestrianRouter.h
 /// @author  Jakob Erdmann
 /// @date    Mon, 03 March 2014
-/// @version $Id: PedestrianRouter.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: PedestrianRouter.h 18096 2015-03-17 09:50:59Z behrisch $
 ///
 // The Pedestrian Router build a special network and (delegegates to a SUMOAbstractRouter)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -86,6 +86,11 @@ struct PedestrianTrip {
         return from->getID() + ":" + to->getID() + ":" + toString(departTime);
     }
 
+
+    inline SUMOVehicleClass getVClass() const {
+        return SVC_PEDESTRIAN;
+    }
+
     const E* from;
     const E* to;
     const N* node; // indicates whether only routing across this node shall be performed
@@ -126,6 +131,7 @@ public:
         std::cout << "initPedestrianNetwork\n";
 #endif
         // build the Pedestrian edges
+        bool haveSeenWalkingArea = false;
         unsigned int numericalID = 0;
         for (size_t i = 0; i < noE; i++) {
             E* edge = E::dictionary(i);
@@ -135,6 +141,7 @@ public:
             } else if (edge->isWalkingArea()) {
                 // only a single edge
                 myEdgeDict.push_back(PedestrianEdge(numericalID++, edge, lane, true));
+                haveSeenWalkingArea = true;
             } else { // regular edge or crossing
                 // forward and backward edges
                 myEdgeDict.push_back(PedestrianEdge(numericalID++, edge, lane, true));
@@ -153,7 +160,7 @@ public:
             if (edge->isInternal() || lane == 0) {
                 continue;
             } else if (edge->isWalkingArea()) {
-                // only a single edge and no connector edges
+                // only a single edge. Connectors are used when routing across a single intersecton
                 myBidiLookup[edge] = std::make_pair(&myEdgeDict[numericalID], &myEdgeDict[numericalID]);
                 myFromToLookup[edge] = std::make_pair(&myEdgeDict[numericalID], &myEdgeDict[numericalID]);
                 numericalID += 1;
@@ -167,32 +174,96 @@ public:
         // build the connections
         for (size_t i = 0; i < noE; i++) {
             E* edge = E::dictionary(i);
-            const L* lane = getSidewalk<E, L>(edge);
-            if (edge->isInternal() || lane == 0) {
+            const L* sidewalk = getSidewalk<E, L>(edge);
+            if (edge->isInternal() || sidewalk == 0) {
                 continue;
             }
             // find all incoming and outgoing lanes for the sidewalk and
             // connect the corresponding PedestrianEdges
-            const L* sidewalk = getSidewalk<E, L>(edge);
             const EdgePair& pair = getBothDirections(edge);
-
 #ifdef PedestrianRouter_DEBUG_NETWORK
             std::cout << "  building connections from " << sidewalk->getID() << "\n";
 #endif
-            std::vector<const L*> outgoing = sidewalk->getOutgoingLanes();
-            for (typename std::vector<const L*>::iterator it = outgoing.begin(); it != outgoing.end(); ++it) {
-                const L* target = *it;
-                const E* targetEdge = &(target->getEdge());
+            if (haveSeenWalkingArea) {
+                std::vector<const L*> outgoing = sidewalk->getOutgoingLanes();
+                // if one of the outgoing lanes is a walking area it must be used.
+                // All other connections shall be ignored
+                bool hasWalkingArea = false;
+                for (typename std::vector<const L*>::iterator it = outgoing.begin(); it != outgoing.end(); ++it) {
+                    const L* target = *it;
+                    const E* targetEdge = &(target->getEdge());
+                    if (targetEdge->isWalkingArea()) {
+                        hasWalkingArea = true;
+                        break;
+                    }
+                }
+                for (typename std::vector<const L*>::iterator it = outgoing.begin(); it != outgoing.end(); ++it) {
+                    const L* target = *it;
+                    const E* targetEdge = &(target->getEdge());
+                    const bool used = (target == getSidewalk<E, L>(targetEdge)
+                                       && (!hasWalkingArea || targetEdge->isWalkingArea()));
 #ifdef PedestrianRouter_DEBUG_NETWORK
-                std::cout << "   lane=" << getSidewalk<E, L>(targetEdge)->getID() << (target == getSidewalk<E, L>(targetEdge) ? "(used)" : "") << "\n";
+                    const L* potTarget = getSidewalk<E, L>(targetEdge);
+                    std::cout << "   lane=" << (potTarget == 0 ? "NULL" : potTarget->getID()) << (used ? "(used)" : "") << "\n";
 #endif
-                if (target == getSidewalk<E, L>(targetEdge)) {
+                    if (used) {
+                        const EdgePair& targetPair = getBothDirections(targetEdge);
+                        pair.first->myFollowingEdges.push_back(targetPair.first);
+                        targetPair.second->myFollowingEdges.push_back(pair.second);
+#ifdef PedestrianRouter_DEBUG_NETWORK
+                        std::cout << "     " << pair.first->getID() << " -> " << targetPair.first->getID() << "\n";
+                        std::cout << "     " << targetPair.second->getID() << " -> " << pair.second->getID() << "\n";
+#endif
+                    }
+                }
+            } else {
+                // we have a network without pedestrian structures. Assume that
+                // all sidewalks at a crossing are interconnected
+                const N* toNode = edge->getToJunction();
+                std::vector<const E*> outgoing = toNode->getOutgoing();
+                for (typename std::vector<const E*>::iterator it = outgoing.begin(); it != outgoing.end(); ++it) {
+                    // build forward and backward connections for all outgoing sidewalks
+                    const E* targetEdge = *it;
+                    const L* target = getSidewalk<E, L>(targetEdge);
+                    if (targetEdge->isInternal() || target == 0) {
+                        continue;
+                    }
                     const EdgePair& targetPair = getBothDirections(targetEdge);
                     pair.first->myFollowingEdges.push_back(targetPair.first);
                     targetPair.second->myFollowingEdges.push_back(pair.second);
 #ifdef PedestrianRouter_DEBUG_NETWORK
                     std::cout << "     " << pair.first->getID() << " -> " << targetPair.first->getID() << "\n";
                     std::cout << "     " << targetPair.second->getID() << " -> " << pair.second->getID() << "\n";
+#endif
+                }
+                std::vector<const E*> incoming = toNode->getIncoming();
+                for (typename std::vector<const E*>::iterator it = incoming.begin(); it != incoming.end(); ++it) {
+                    // build forward-to-backward connections for all incoming sidewalks
+                    const E* targetEdge = *it;
+                    const L* target = getSidewalk<E, L>(targetEdge);
+                    if (targetEdge->isInternal() || target == 0 || targetEdge == edge) {
+                        continue;
+                    }
+                    const EdgePair& targetPair = getBothDirections(targetEdge);
+                    pair.first->myFollowingEdges.push_back(targetPair.second); // change direction
+#ifdef PedestrianRouter_DEBUG_NETWORK
+                    std::cout << "     " << pair.first->getID() << " -> " << targetPair.second->getID() << "\n";
+#endif
+
+                }
+                const N* fromNode = edge->getFromJunction();
+                outgoing = fromNode->getOutgoing();
+                for (typename std::vector<const E*>::iterator it = outgoing.begin(); it != outgoing.end(); ++it) {
+                    // build backward-to-forward connections for all outgoing sidewalks at the fromNode
+                    const E* targetEdge = *it;
+                    const L* target = getSidewalk<E, L>(targetEdge);
+                    if (targetEdge->isInternal() || target == 0 || targetEdge == edge) {
+                        continue;
+                    }
+                    const EdgePair& targetPair = getBothDirections(targetEdge);
+                    pair.second->myFollowingEdges.push_back(targetPair.first);
+#ifdef PedestrianRouter_DEBUG_NETWORK
+                    std::cout << "     " << pair.second->getID() << " -> " << targetPair.first->getID() << "\n";
 #endif
                 }
             }
@@ -271,8 +342,13 @@ public:
         return (unsigned int)myFollowingEdges.size();
     }
 
-    PedestrianEdge* getSuccessor(unsigned int i) const {
-        return myFollowingEdges[i];
+    const std::vector<PedestrianEdge*>& getSuccessors() const {
+        return myFollowingEdges;
+    }
+
+    const std::vector<PedestrianEdge*>& getSuccessors(SUMOVehicleClass /*vClass*/) const {
+        // the network is already tailored for pedestrians. No need to check for permissions here
+        return myFollowingEdges;
     }
 
     bool prohibits(const PedestrianTrip<E, N>* const trip) const {
@@ -320,7 +396,7 @@ public:
 
         }
 #ifdef PedestrianRouter_DEBUG_EFFORTS
-        std::cout << " effort for " << getID() << " at " << time << ": " << length / trip->speed + tlsDelay << " l=" << length << " s=" << trip->speed << " tlsDelay=" << tlsDelay << "\n";
+        std::cout << " effort for " << trip->getID() << " at " << time << " edge=" << edge->getID() << " effort=" << length / trip->speed + tlsDelay << " l=" << length << " s=" << trip->speed << " tlsDelay=" << tlsDelay << "\n";
 #endif
         return length / trip->speed + tlsDelay;
     }
@@ -397,6 +473,14 @@ public:
     void compute(const E* from, const E* to, SUMOReal departPos, SUMOReal arrivalPos, SUMOReal speed,
                  SUMOTime msTime, const N* onlyNode, std::vector<const E*>& into, bool allEdges = false) {
         //startQuery();
+        if (getSidewalk<E, L>(from) == 0) {
+            WRITE_WARNING("Departure edge '" + from->getID() + "' does not allow pedestrians.");
+            return;
+        }
+        if (getSidewalk<E, L>(to) == 0) {
+            WRITE_WARNING("Destination edge '" + to->getID() + "' does not allow pedestrians.");
+            return;
+        }
         _PedestrianTrip trip(from, to, departPos, arrivalPos, speed, msTime, onlyNode);
         std::vector<const _PedestrianEdge*> intoPed;
         myInternalRouter->compute(_PedestrianEdge::getDepartEdge(from),
@@ -407,12 +491,17 @@ public:
             }
         }
 #ifdef PedestrianRouter_DEBUG_ROUTES
+        SUMOReal time = msTime;
+        for (size_t i = 0; i < intoPed.size(); ++i) {
+            time += myInternalRouter->getEffort(intoPed[i], &trip, time);
+        }
         std::cout << TIME2STEPS(msTime) << " trip from " << from->getID() << " to " << to->getID()
                   << " departPos=" << departPos
                   << " arrivalPos=" << arrivalPos
                   << " onlyNode=" << (onlyNode == 0 ? "NULL" : onlyNode->getID())
                   << " edges=" << toString(intoPed)
                   << " resultEdges=" << toString(into)
+                  << " time=" << time
                   << "\n";
 #endif
         //endQuery();
