@@ -9,10 +9,10 @@ function setRoute(vehID, edgeList)
 % 
 %   changes the route for vehicle id 1 to edges 1-2-4-6-7.
 
-%   Copyright 2013 Universidad Nacional de Colombia,
+%   Copyright 2015 Universidad Nacional de Colombia,
 %   Politecnico Jaime Isaza Cadavid.
 %   Authors: Andres Acosta, Jairo Espinosa, Jorge Espinosa.
-%   $Id: setRoute.m 2 2013-12-21 21:39:57Z aacosta $
+%   $Id: setRoute.m 20 2015-03-02 16:52:32Z afacostag $
 
 import traci.constants
 global message
@@ -20,9 +20,9 @@ global message
 traci.beginMessage(constants.CMD_SET_VEHICLE_VARIABLE, constants.VAR_ROUTE, vehID,...
     1+4+sum(cellfun('length', edgeList))+4*length(edgeList));
 message.string = [message.string uint8(sscanf(constants.TYPE_STRINGLIST,'%x')) ...
-    fliplr(typecast(int32(length(edgeList)),'uint8'))];
+    traci.packInt32(length(edgeList))];
 for i=1:length(edgeList)
-    message.string = [message.string fliplr(typecast(int32(length(edgeList{i})),'uint8')) ...
+    message.string = [message.string traci.packInt32(length(edgeList{i})) ...
         uint8(edgeList{i})];
 end
 traci.sendExact()
\ No newline at end of file
