@@ -4,10 +4,10 @@ function close()
 %   connections have to be closed, they must be enabled through the
 %   traci.switch() command, and closed with CLOSE().
 
-%   Copyright 2013 Universidad Nacional de Colombia,
+%   Copyright 2015 Universidad Nacional de Colombia,
 %   Politecnico Jaime Isaza Cadavid.
 %   Authors: Andres Acosta, Jairo Espinosa, Jorge Espinosa.
-%   $Id: close.m 2 2013-12-21 21:39:57Z aacosta $
+%   $Id: close.m 20 2015-03-02 16:52:32Z afacostag $
 
 global connections message
 import traci.constants
@@ -22,6 +22,7 @@ traci.sendExact();
 
 % Close and clear the tcp object
 if isKey(connections,'')
-    fclose(connections(''));
+	activeConnection = connections('');
+    activeConnection.socket.close();
     clear connections('')
 end
\ No newline at end of file
