@@ -9,7 +9,7 @@
 %   Copyright 2013 Universidad Nacional de Colombia,
 %   Politecnico Jaime Isaza Cadavid.
 %   Authors: Andres Acosta, Jairo Espinosa, Jorge Espinosa.
-%   $Id: traci_test2.m 3 2013-12-22 00:07:52Z aacosta $
+%   $Id: traci_test2.m 20 2015-03-02 16:52:32Z afacostag $
 
 clear all
 close all
@@ -23,7 +23,21 @@ clc
 % interface mode. You have to set the SUMO_HOME environment variable
 % pointing to your SUMO root directory and modify the windows path to 
 % include the %SUMO_HOME%/bin directory.
-system(['sumo-gui -c ' getenv('SUMO_HOME') '\docs\tutorial\traci_tls\data\cross.sumocfg&']);
+
+try
+	system(['sumo-gui -c ' getenv('SUMO_HOME') '\docs\tutorial\traci_tls\data\cross.sumocfg&']);
+catch err
+end
+
+
+% To test sine vehicle commands, we have to check wether the sumo 0.20.0
+% version is installed, because in that version the prefix of the vehicle
+% names has changed.
+if isempty(strfind(getenv('SUMO_HOME'),'sumo-0.21.0'))
+	testVehicle = '10';
+else
+	testVehicle = 'right_10';
+end
 
 import traci.constants
 
@@ -42,11 +56,18 @@ WEYELLOW = 'ryry';
 PROGRAM = {WEYELLOW,WEYELLOW,WEYELLOW,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSGREEN,NSYELLOW,NSYELLOW,WEGREEN};
 
 programPointer = length(PROGRAM);
-steps = zeros(1,800);
+% step = 0;
+% steps = zeros(1,800);
 
 
 %% GETIDLIST COMMANDS
 
+
+% areal_dets = traci.areal.getIDList();
+% fprintf('IDs of the areal detectors in the simulation:\n')
+% for i=1:length(areal_dets)
+%     fprintf('%s\n',areal_dets{i});
+% end
 % edges = traci.edge.getIDList();
 % fprintf('IDs of the edges in the simulation:\n')
 % for i=1:length(edges)
@@ -105,11 +126,11 @@ steps = zeros(1,800);
 % LOADED IN THE NETWORK, IN THE MAIN LOOP
 
 
-%% SUBSCIBE COMMANDS
+%% SUBSCIBE COMMANDS: Note that you have to create the required detectors in the cross.det.xml file
 
 % traci.edge.subscribe('1i');
 % traci.gui.subscribe('View #0');
-traci.inductionloop.subscribe('0');
+% traci.inductionloop.subscribe('0');
 % traci.junction.subscribe('0');
 % traci.lane.subscribe('1i_0');
 % traci.multientryexit.subscribe('e3_0_1i');
@@ -118,8 +139,8 @@ traci.inductionloop.subscribe('0');
 % traci.trafficlights.subscribe('0',{constants.TL_RED_YELLOW_GREEN_STATE});
 % tlsCurrentPhaseHandle = traci.trafficlights.getSubscriptionResults('0');
 % tlsCurrentPhase = tlsCurrentPhaseHandle(constants.TL_RED_YELLOW_GREEN_STATE);
-% fprintf('Fase del semaforo: %s\n', tlsCurrentPhase)
-% traci.vehicle.subscribe('5');
+% fprintf('Traffic lights phase: %s\n', tlsCurrentPhase)
+% traci.vehicle.subscribe(testVehicle);
 % traci.vehicletype.subscribe('typeWE');
 % maxSpeedWEHandle = traci.vehicletype.getSubscriptionResults('typeWE');
 % maxSpeedWE = maxSpeedWEHandle(constants.VAR_MAXSPEED);
@@ -140,8 +161,8 @@ traci.inductionloop.subscribe('0');
 % traci.poi.setPosition('mypoi', 550, 580);
 % poiposition = traci.poi.getPosition('mypoi')
 % traci.poi.setColor('mypoi', [255 255 255 0]);
-% traci.poi.remove('mypoi', 1);
-
+% % traci.poi.remove('mypoi', 1);
+% 
 % traci.polygon.add('mypolygon', {[440,440],[440,450],[450,440],[450,450]},...
 %     [0 255 255 0], false, '', 1);
 % traci.polygon.add('my2ndpolygon', {[400,400],[400,420],[430,420],[430,400],[400,400]},...
@@ -162,7 +183,7 @@ traci.inductionloop.subscribe('0');
 
 
 %% CONTEXT SUBSCRIPTIONS
-% It's worth noting that, accordong to
+% It's worth noting that, according to
 % http://sumo-sim.org/userdoc/TraCI/Object_Context_Subscription.html, only
 % the following SUMO objects are supported: inductive loops, lanes,
 % vehicles, points-of-interest, polygons, junctions, edges.
@@ -173,25 +194,25 @@ traci.inductionloop.subscribe('0');
 % traci.lane.subscribeContext('4i_0',constants.CMD_GET_LANE_VARIABLE,50);
 % traci.poi.subscribeContext('mypoi',constants.CMD_GET_VEHICLE_VARIABLE,50);
 % traci.polygon.subscribeContext('mypolygon',constants.CMD_GET_POLYGON_VARIABLE,30);
-% traci.vehicle.subscribeContext('5',constants.CMD_GET_VEHICLE_VARIABLE,30);
-
-WElaneoccupancy = zeros(1,800);
-NSlaneoccupancy = zeros(1,800);
+% traci.vehicle.subscribeContext(testVehicle,constants.CMD_GET_VEHICLE_VARIABLE,10);
+% 
+% WElaneoccupancy = zeros(1,800);
+% NSlaneoccupancy = zeros(1,800);
 
 % getminexectednumber returns the number of expected vehicles to leave the
 % network, which will be the condition to execute the simulation.
 % MinExpectedNumber = traci.simulation.getMinExpectedNumber();
 
 %% GUI SET COMMANDS
-traci.gui.setZoom('View #0', 1000);
-traci.gui.setOffset('View #0',  523.7211,  525.9342);
+% traci.gui.setZoom('View #0', 1000);
+% traci.gui.setOffset('View #0',  523.7211,  525.9342);
 traci.gui.setSchema('View #0',  'real world');
 % traci.gui.setBoundary('View #0', 386.95, 485.88, 651.64, 589.01);
-traci.gui.trackVehicle('View #0', '10');
+% traci.gui.trackVehicle('View #0', testVehicle);
 
 %% LANE SET COMMANDS
-% traci.lane.setAllowed('1i_0',{'typeWE'});
-% traci.lane.setDisallowed('1i_0',{'typeWE'});
+% traci.lane.setAllowed('1i_0',{'unknown'});
+% traci.lane.setDisallowed('1i_0',{'unknown'});
 % traci.lane.setMaxSpeed('1i_0',5);
 % traci.lane.setLength('1i_0',450);
 
@@ -200,36 +221,36 @@ traci.gui.trackVehicle('View #0', '10');
 % routeUpEdges = traci.route.getEdges('up')
 
 %% VEHICLE SET COMMANDS
-% traci.vehicle.setMaxSpeed('5',5);
-% traci.vehicle.setStop('5','1i',50,0,40000);
-% traci.vehicle.changeLane('5',0,40000);
-% traci.vehicle.slowDown('5',1,180000);
-% traci.vehicle.changeTarget('5','2o');
-% traci.vehicle.setRouteID('5','down');
-% traci.vehicle.setRoute('5',{'51o' '1i' '2o'});
-% traci.vehicle.setAdaptedTraveltime('5',10000,50000,'1i',15000);
-% traci.vehicle.setEffort('5',10000,50000,'1i',12.454);
-% traci.vehicle.rerouteTraveltime('5');
-% traci.vehicle.rerouteEffort('5');
-% traci.vehicle.setSignals('5',2);
-traci.vehicle.setSpeed('10',5);
-traci.vehicle.setColor('10',[0 0 255 0]);
-traci.vehicle.setLength('10',10);
-% traci.vehicle.setVehicleClass('5','unknown');
-% traci.vehicle.setSpeedFactor('5',0.6);
-% traci.vehicle.setSpeedDeviation('5',0.02);
-% traci.vehicle.setEmissionClass('5','unknown');
-% traci.vehicle.setWidth('5',3);
-traci.vehicle.setMinGap('10',10);
-% traci.vehicle.setShapeClass('5','');
-traci.vehicle.setAccel('10',2);
-traci.vehicle.setDecel('10',2);
-traci.vehicle.setImperfection('10',1);
-traci.vehicle.setTau('10',1);
+% traci.vehicle.setMaxSpeed(testVehicle,5);
+% traci.vehicle.setStop(testVehicle,'1i',50,0,40000);
+% traci.vehicle.changeLane(testVehicle,0,40000);
+% traci.vehicle.slowDown(testVehicle,1,180000);
+% traci.vehicle.changeTarget(testVehicle,'2o');
+% traci.vehicle.setRouteID(testVehicle,'down');
+% traci.vehicle.setRoute(testVehicle,{'51o' '1i' '2o'});
+% traci.vehicle.setAdaptedTraveltime(testVehicle,10000,50000,'1i',15000);
+% % traci.vehicle.setEffort(testVehicle,10000,50000,'1i',0.125); %Not online
+% % traci.vehicle.rerouteTraveltime(testVehicle); %Not online
+% % traci.vehicle.rerouteEffort(testVehicle); %Not online
+% traci.vehicle.setSignals(testVehicle,2);
+% traci.vehicle.setSpeed(testVehicle,5);
+% traci.vehicle.setColor(testVehicle,[0 0 255 0]);
+% traci.vehicle.setLength(testVehicle,10);
+% traci.vehicle.setVehicleClass(testVehicle,'unknown');
+% traci.vehicle.setSpeedFactor(testVehicle,0.6);
+% % traci.vehicle.setSpeedDeviation(testVehicle,5); %Not online
+% traci.vehicle.setEmissionClass(testVehicle,'unknown');
+% traci.vehicle.setWidth(testVehicle,3);
+% traci.vehicle.setMinGap(testVehicle,10);
+% traci.vehicle.setShapeClass(testVehicle,'');
+% traci.vehicle.setAccel(testVehicle,2);
+% traci.vehicle.setDecel(testVehicle,2);
+% traci.vehicle.setImperfection(testVehicle,1);
+% traci.vehicle.setTau(testVehicle,1);
 % traci.vehicle.add('myvehicle','down');
 % traci.vehicle.remove('myvehicle');
 % traci.gui.trackVehicle('View #0', 'myvehicle');
-% traci.vehicle.moveToVTD('10','2o',0,608,509);
+% traci.vehicle.moveToVTD('right_10','2o',0,608,509);
 
 %% VEHICLE TYPE COMMANDS
 
@@ -246,8 +267,8 @@ traci.vehicle.setTau('10',1);
 % typeShapeClass = traci.vehicletype.getShapeClass('typeWE')
 % typeMinGap = traci.vehicletype.getShapeClass('typeWE')
 % typeWidth = traci.vehicletype.getWidth('typeWE')
-% typeColor = traci.vehicletype.getColor('typeWE
-
+% typeColor = traci.vehicletype.getColor('typeWE')
+% 
 % traci.vehicletype.setLength('typeWE',8);
 % traci.vehicletype.setMaxSpeed('typeWE',10);
 % traci.vehicletype.setVehicleClass('typeWE','passenger');
@@ -263,44 +284,48 @@ traci.vehicle.setTau('10',1);
 % traci.vehicletype.setTau('typeWE',0.1);
 % traci.vehicletype.setColor('typeWE',[255 255 255 0]);
 
-for i=1:length(steps)
-    
-    % Perform a simulation step (one second)
-    traci.simulationStep();
+step = 1;
+% for i=1:length(steps)
+while traci.simulation.getMinExpectedNumber()>0
+    % Here, we demonstrate how to use the simulationStep command using an
+    % argument. In this case, the simulation is performed each 5 seconds,
+    % note the behavior when you increase the delay in the gui
+    traci.simulationStep(5000*step);
+    pause(1);
     programPointer = min(programPointer+1, length(PROGRAM));
     
     % Get the number of vehicles that passed through the induction loop in
     % the last simulation step
-    % no = traci.inductionloop.getLastStepVehicleNumber('0');
+    numPriorityVehicles = traci.inductionloop.getLastStepVehicleNumber('0');
     
     % SHOW THE VEHICLES IDS INSIDE THE NETWORK
-    % vehicles = traci.vehicle.getIDList();
+     vehicles = traci.vehicle.getIDList();
     % fprintf('IDs of the vehicles in the simulation\n')
     % for j=1:length(vehicles)
     %     fprintf('%s\n',vehicles{j});
     % end
     
-    % Subscribe to the vehicle with ID '5' when it is loaded in the
-    % network
-    % if ismember('5',vehicles)
-    %     vehicle5Handle = traci.vehicle.getSubscriptionResults('5');
-    %     vehicle5 = {vehicle5Handle(constants.VAR_ROAD_ID) vehicle5Handle(constants.VAR_LANEPOSITION)};
+    % Subscribe to the vehicle with the id contained in the variable "testVehicle" 
+	% when it is loaded in the network
+    % if ismember(testVehicle,vehicles)
+    %     testVehicleHandle = traci.vehicle.getSubscriptionResults(testVehicle);
+    %     testVehicleHandle = {testVehicleHandle(constants.VAR_ROAD_ID) testVehicleHandle(constants.VAR_LANEPOSITION)};
     % end
     
-    %% GETSUBSCRIPTIONRESULTS COMMANDS
+    %% GETSUBSCRIPTIONRESULTS COMMANDS: Note that you have to create the required detectors in the cross.det.xml file
     
-    % occupancyEdge1Handle = traci.edge.getSubscriptionResults('1i');
-    % WElaneoccupancy(i) = occupancyEdge1Handle(constants.LAST_STEP_VEHICLE_NUMBER);
+%     occupancyEdge1Handle = traci.edge.getSubscriptionResults('1i');
+%     WElaneoccupancy(i) = occupancyEdge1Handle(constants.LAST_STEP_VEHICLE_NUMBER);
     % offsethandle = traci.gui.getSubscriptionResults('View #0');
-    % offset = offsethandle(traci.VAR_VIEW_OFFSET);
-    indloopSubsResults = traci.inductionloop.getSubscriptionResults('0');
-    no = indloopSubsResults(constants.LAST_STEP_VEHICLE_NUMBER);
+    % offset = offsethandle(traci.constants.VAR_VIEW_OFFSET);
+    % indloopSubsResults = traci.inductionloop.getSubscriptionResults('0');
+    % no = indloopSubsResults(constants.LAST_STEP_VEHICLE_NUMBER);
     % junctionPositionHandle = traci.junction.getSubscriptionResults('0');
     % junctionPosition = junctionPositionHandle(constants.VAR_POSITION);
-    % occupancyLane1Handle = traci.lane.getSubscriptionResults('1i_0');
-    % WElaneoccupancy(i) = occupancyLane1Handle(constants.LAST_STEP_VEHICLE_NUMBER);
+%     occupancyLane1Handle = traci.lane.getSubscriptionResults('1i_0');
+%     WElaneoccupancy(i) = occupancyLane1Handle(constants.LAST_STEP_VEHICLE_NUMBER);
     % occupancyLane1Handle = traci.multientryexit.getSubscriptionResults('e3_0_1i');
-    % WElaneoccupancy(i) = occupancyLane1Handle(constants.LAST_STEP_VEHICLE_NUMBER);
+%     WElaneoccupancy(i) = occupancyLane1Handle(constants.LAST_STEP_VEHICLE_NUMBER);
     % poiPositionHandle = traci.poi.getSubscriptionResults('mypoi');
     % poiPosition = poiPositionHandle(constants.VAR_POSITION);
     % polygonPositionHandle = traci.polygon.getSubscriptionResults('mypolygon');
@@ -309,8 +334,6 @@ for i=1:length(steps)
     % routeList = routeListHandle(constants.ID_LIST);
     % departedVehicleIDsHandle = traci.simulation.getSubscriptionResults();
     % departedVehicleIDs = departedVehicleIDsHandle(constants.VAR_DEPARTED_VEHICLES_IDS);
-    % tlsCurrentPhaseHandle = traci.trafficlights.getSubscriptionResults('0');
-    % tlsCurrentPhase = tlsCurrentPhaseHandle(constants.TL_CURRENT_PHASE);
     
     %% GET CONTEXT SUBSCRIPTION RESULTS COMMANDS
     
@@ -329,8 +352,25 @@ for i=1:length(steps)
     % laneContextSubscriptionResults = traci.lane.getContextSubscriptionResults('4i_0');
     % poiContextSubscriptionResults = traci.poi.getContextSubscriptionResults('mypoi');
     % polygonContextSubscriptionResults = traci.polygon.getContextSubscriptionResults('mypolygon');
-    % vehicleContextSubscriptionResults = traci.vehicle.getContextSubscriptionResults('5');
+    % vehicleContextSubscriptionResults = traci.vehicle.getContextSubscriptionResults(testVehicle);
     
+	%% AREAL DETECTOR COMMANDS: Note that you have to create the detector in the cross.det.xml file
+ 	% arealDetectorIDCount = traci.areal.getIDCount();
+ 	% fprintf('Number of areal detectors in the simulation: %d\n',arealDetectorIDCount);
+	
+	% JamLengthVehicle = traci.areal.getJamLengthVehicle('0');
+	% fprintf('Jam lenght in vehicles in the areal detector 0: %d\n',JamLengthVehicle);
+	
+	% JamLengthMeters = traci.areal.getJamLengthMeters('0');
+	% fprintf('Jam lenght in meters in the areal detector 0: %d\n',JamLengthMeters);
+	
+	% vehicleMeanSpeedAreal0 = traci.areal.getLastStepMeanSpeed('0');
+    % fprintf('Average speed in the areal detector 0: %d\n',vehicleMeanSpeedAreal0);
+	
+	% vehicleOccupancyAreal0 = traci.areal.getLastStepOccupancy('0');
+    % fprintf('Occupancy in the areal detector 0 1i: %d\n',vehicleOccupancyAreal0);
+	
+
     %% EDGE COMMANDS
     
     % edgeIDCount = traci.edge.getIDCount();
@@ -357,7 +397,7 @@ for i=1:length(steps)
     % NOxEmissionEdge1i = traci.edge.getNOxEmission('1i');
     % fprintf('NOx emission in the edge 1i: %d\n',NOxEmissionEdge1i);
     
-    % fuelConsumptionEdge1i = traci.edge.getFuelConsumptionEmission('1i');
+    % fuelConsumptionEdge1i = traci.edge.getFuelConsumption('1i');
     % fprintf('Fuel consumption in the edge 1i: %d\n',fuelConsumptionEdge1i);
     
     % noiseEmissionEdge1i = traci.edge.getNoiseEmission('1i');
@@ -434,7 +474,7 @@ for i=1:length(steps)
     % lane1i0HalringNumber = traci.lane.getLastStepHaltingNumber('1i_0')
     % lane1i0VehicleIDs = traci.lane.getLastStepVehicleIDs('1i_0')
     
-    %% MULTIENTRY=EXIT COMMANDS
+    %% MULTIENTRY=EXIT COMMANDS: Note that you have to create the detector in the cross.det.xml file
     
     % muiVehicleNumber = traci.multientryexit.getLastStepVehicleNumber('e3_0_1i')
     % muiMeanSpeed = traci.multientryexit.getLastStepMeanSpeed('e3_0_1i')
@@ -449,8 +489,8 @@ for i=1:length(steps)
     % loadedIDList = traci.simulation.getLoadedIDList();
     % departedNumber = traci.simulation.getDepartedNumber()
     % departedIDList = traci.simulation.getDepartedIDList();
-    % if ismember('5',departedIDList)
-    %     traci.vehicle.moveTo('5','1i_0',20);
+    % if ismember(testVehicle,departedIDList)
+    %     traci.vehicle.moveTo(testVehicle,'1i_0',20);
     % end
     % arrivedNumber = traci.simulation.getArrivedNumber()
     % arrivedIDList = traci.simulation.getArrivedIDList()
@@ -476,61 +516,64 @@ for i=1:length(steps)
     % traci.trafficlights.setPhase('0',0);
     % traci.trafficlights.setProgram('0','0');
     % traci.trafficlights.setPhaseDuration('0',5);
-    myRYGDefinition = traci.trafficlights.Logic('0',0,0,0,...
-        {traci.trafficlights.Phase(31000,31000,31000,'GrGr'),...
-        traci.trafficlights.Phase(31000,31000,31000,'rGrG'),...
-        traci.trafficlights.Phase(6000,6000,6000,'ryry')});
-    traci.trafficlights.setCompleteRedYellowGreenDefinition('0',myRYGDefinition);
+%     myRYGDefinition = traci.trafficlights.Logic('0',0,0,0,...
+%         {traci.trafficlights.Phase(31000,31000,31000,'GrGr'),...
+%         traci.trafficlights.Phase(31000,31000,31000,'rGrG'),...
+%         traci.trafficlights.Phase(6000,6000,6000,'ryry')});
+    % traci.trafficlights.setCompleteRedYellowGreenDefinition('0',tlsRYGDefinition{1});
     % tlsRYGDefinition = traci.trafficlights.getCompleteRedYellowGreenDefinition('0');
     
     %% VEHICLE GET COMMANDS
     
-    % vehSpeed = traci.vehicle.getSpeed('5')
-    % vehSpeedWOTraci = traci.vehicle.getSpeedWithoutTraCI('5')
-    % vehPosition = traci.vehicle.getPosition('5')
-    % vehAngle = traci.vehicle.getAngle('5')
-    % vehRoadID = traci.vehicle.getRoadID('5')
-    % vehLaneID = traci.vehicle.getLaneID('5')
-    % vehLaneIndex = traci.vehicle.getLaneIndex('5')
-    % vehTypeID = traci.vehicle.getTypeID('5')
-    % vehRouteID = traci.vehicle.getRouteID('5')
-    % vehRoute = traci.vehicle.getRoute('5')
-    % vehLanePos = traci.vehicle.getLanePosition('5')
-    % vehColor = traci.vehicle.getColor('5')
-    % vehCO2Emission = traci.vehicle.getCO2Emission('5')
-    % vehCOEmission = traci.vehicle.getCOEmission('5')
-    % vehPmxEmission = traci.vehicle.getPMxEmission('5')
-    % vehNOxEmission = traci.vehicle.getNOxEmission('5')
-    % vehFuelConsumption = traci.vehicle.getFuelConsumption('5')
-    % vehAdaptedTraveltime = traci.vehicle.getAdaptedTraveltime('5',10,'1i')
-    % vehEffort = traci.vehicle.getEffort('5',10,'1i')
-    % vehValidRoute = traci.vehicle.isRouteValid('5')
-    % vehSignals = traci.vehicle.getSignals('5')
-    % vehMaxSpeed = traci.vehicle.getMaxSpeed('5')
-    % vehClass = traci.vehicle.getVehicleClass('5')
-    % vehSpeedFactor = traci.vehicle.getSpeedFactor('5')
-    % vehSpeedDeviation = traci.vehicle.getSpeedDeviation('5')
-    % vehEmissionClass = traci.vehicle.getEmissionClass('5')
-    % vehWidth = traci.vehicle.getWidth('5')
-    % vehMinGap = traci.vehicle.getMinGap('5')
-    % vehShapeClass = traci.vehicle.getShapeClass('5')
-    % vehAccel = traci.vehicle.getAccel('5')
-    % vehDecel = traci.vehicle.getDecel('5')
-    % vehImperfection = traci.vehicle.getImperfection('5')
-    % vehTau = traci.vehicle.getTau('5')
-    % vehBestLanes = traci.vehicle.getBestLanes('5')
-    % vehDrivingDistance = traci.vehicle.getDrivingDistance('5','2o',30)
-    % vehDrivingDistance2D = traci.vehicle.getDrivingDistance2D('5',620,510)
+    % vehSpeed = traci.vehicle.getSpeed(testVehicle)
+    % vehSpeedWOTraci = traci.vehicle.getSpeedWithoutTraCI(testVehicle)
+    % vehPosition = traci.vehicle.getPosition(testVehicle)
+    % vehAngle = traci.vehicle.getAngle(testVehicle)
+    % vehRoadID = traci.vehicle.getRoadID(testVehicle)
+    % vehLaneID = traci.vehicle.getLaneID(testVehicle)
+    % vehLaneIndex = traci.vehicle.getLaneIndex(testVehicle)
+    % vehTypeID = traci.vehicle.getTypeID(testVehicle)
+    % vehRouteID = traci.vehicle.getRouteID(testVehicle)
+    % vehRoute = traci.vehicle.getRoute(testVehicle)
+    % vehLanePos = traci.vehicle.getLanePosition(testVehicle)
+    % vehColor = traci.vehicle.getColor(testVehicle)
+    % vehCO2Emission = traci.vehicle.getCO2Emission(testVehicle)
+    % vehCOEmission = traci.vehicle.getCOEmission(testVehicle)
+    % vehPmxEmission = traci.vehicle.getPMxEmission(testVehicle)
+    % vehNOxEmission = traci.vehicle.getNOxEmission(testVehicle)
+    % vehFuelConsumption = traci.vehicle.getFuelConsumption(testVehicle)
+    % vehAdaptedTraveltime = traci.vehicle.getAdaptedTraveltime(testVehicle,10,'1i')
+    % vehEffort = traci.vehicle.getEffort(testVehicle,10,'1i')
+    % vehValidRoute = traci.vehicle.isRouteValid(testVehicle)
+    % vehSignals = traci.vehicle.getSignals(testVehicle)
+    % vehMaxSpeed = traci.vehicle.getMaxSpeed(testVehicle)
+    % vehClass = traci.vehicle.getVehicleClass(testVehicle)
+    % vehSpeedFactor = traci.vehicle.getSpeedFactor(testVehicle)
+    % vehSpeedDeviation = traci.vehicle.getSpeedDeviation(testVehicle)
+    % vehEmissionClass = traci.vehicle.getEmissionClass(testVehicle)
+    % vehWidth = traci.vehicle.getWidth(testVehicle)
+    % vehMinGap = traci.vehicle.getMinGap(testVehicle)
+    % vehShapeClass = traci.vehicle.getShapeClass(testVehicle)
+    % vehAccel = traci.vehicle.getAccel(testVehicle)
+    % vehDecel = traci.vehicle.getDecel(testVehicle)
+    % vehImperfection = traci.vehicle.getImperfection(testVehicle)
+    % vehTau = traci.vehicle.getTau(testVehicle)
+    if ismember(testVehicle,vehicles)
+        vehLeader = traci.vehicle.getLeader(testVehicle, 1)
+    end
+    % vehBestLanes = traci.vehicle.getBestLanes(testVehicle)
+    % vehDrivingDistance = traci.vehicle.getDrivingDistance(testVehicle,'2o',30)
+    % vehDrivingDistance2D = traci.vehicle.getDrivingDistance2D(testVehicle,620,510)
       
     
     % Change the phase of the traffic light if a vehicle passed through the
     % induction loop
-    if no > 0
+    if numPriorityVehicles > 0
         % traci.gui.screenshot('View #0','passedvehicle.bmp')
         % loop0VehicleData = traci.inductionloop.getVehicleData('0')
-        if programPointer == length(PROGRAM)-1
+        if programPointer == length(PROGRAM)
             programPointer = 1;
-        else
+		elseif ~strcmp(PROGRAM(programPointer), WEYELLOW)
             programPointer = 4;
         end
     end
@@ -541,24 +584,25 @@ for i=1:length(steps)
     % if no > 0
     %     tlsCurrentPhaseHandle = traci.trafficlights.getSubscriptionResults('0');
     %     tlsCurrentPhase = tlsCurrentPhaseHandle(constants.TL_RED_YELLOW_GREEN_STATE);
-    %     fprintf('La fase del semaforo cambio a: %s\n', tlsCurrentPhase)
+    %     fprintf('The traffic lights'' phase changed to: %s\n', tlsCurrentPhase)
     % end
     
     % AN ADDITIONAL EVIDENCE OF THE LANE SUBSCRIPTIONS, ENABLE THE PLOTTING
     % FUNCTIONS BELOW TO VISUALIZE IT.
-    WElaneoccupancy(i) = traci.lane.getLastStepVehicleNumber('1i_0')+...
-        traci.lane.getLastStepVehicleNumber('2i_0');
-    NSlaneoccupancy(i) = traci.lane.getLastStepVehicleNumber('3i_0')+...
-        traci.lane.getLastStepVehicleNumber('4i_0');
-    
-    steps(i) = i;
-    MinExpectedNumber = traci.simulation.getMinExpectedNumber();
+%     WElaneoccupancy(i) = traci.lane.getLastStepVehicleNumber('1i_0')+...
+%         traci.lane.getLastStepVehicleNumber('2i_0');
+%     NSlaneoccupancy(i) = traci.lane.getLastStepVehicleNumber('3i_0')+...
+%         traci.lane.getLastStepVehicleNumber('4i_0');
+% %     
+%     steps(i) = i;
+%     MinExpectedNumber = traci.simulation.getMinExpectedNumber();
+    step = step + 1;
 end
 traci.close()
-plot(steps, WElaneoccupancy)
-hold;
-plot(steps, NSlaneoccupancy, 'r')
-legend('WE lane occupancy', 'NS lane occupancy')
-title('Lane occupancy vs time')
-xlabel('t (seconds)')
-ylabel('number of vehicles')
\ No newline at end of file
+% plot(steps, WElaneoccupancy)
+% hold;
+% plot(steps, NSlaneoccupancy, 'r')
+% legend('WE lane occupancy', 'NS lane occupancy')
+% title('Lane occupancy vs time')
+% xlabel('t (seconds)')
+% ylabel('number of vehicles')
\ No newline at end of file
