@@ -6,14 +6,14 @@
 @author  Laura Bieker
 @author  Jakob Erdmann
 @date    2007-03-20
-@version $Id: netcheck.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: netcheck.py 18096 2015-03-17 09:50:59Z behrisch $
 
 This script does simple check for the network.
 It tests whether the network is (weakly) connected.
 It needs one parameter, the SUMO net (.net.xml).
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2007-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2007-2015 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -21,24 +21,26 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
-import os, sys
+import os
+import sys
 from optparse import OptionParser
 sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 import sumolib.net
 
+
 def parse_args():
     USAGE = "Usage: " + sys.argv[0] + " <net> <options>"
     optParser = OptionParser()
     optParser.add_option("-s", "--source",
-            default=False, help="List edges reachable from the source")
-    optParser.add_option("--selection-output", 
-            help="When used with --source, write output to file as a loadable selection")
+                         default=False, help="List edges reachable from the source")
+    optParser.add_option("--selection-output",
+                         help="When used with --source, write output to file as a loadable selection")
 
     options, args = optParser.parse_args()
     if len(args) != 1:
         sys.exit(USAGE)
     options.net = args[0]
-    return options 
+    return options
 
 
 def getWeaklyConnected(net):
@@ -80,7 +82,7 @@ def getReachable(net, source_id, options):
         fringe = new_fringe
 
     print "%s of %s edges are reachable from edge '%s':" % (
-            len(found), len(net.getEdges()), source_id)
+        len(found), len(net.getEdges()), source_id)
 
     if options.selection_output:
         with open(options.selection_output, 'w') as f:
@@ -104,4 +106,3 @@ if __name__ == "__main__":
                 print "Component", idx
                 print " ".join(comp)
                 print
-
