@@ -4,12 +4,12 @@
 @author  Jakob Erdmann
 @author  Michael Behrisch
 @date    2012-12-04
-@version $Id: __init__.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: __init__.py 18096 2015-03-17 09:50:59Z behrisch $
 
 This file includes all file descriptions.
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -18,6 +18,5 @@ the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
 
-import additional, selection
-
-
+import additional
+import selection
