@@ -3,13 +3,13 @@
 @author  Daniel Krajzewicz
 @author  Michael Behrisch
 @date    2013-01-15
-@version $Id: gpsdat.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: gpsdat.py 18096 2015-03-17 09:50:59Z behrisch $
 
 This module includes functions for converting SUMO's fcd-output into
 data files read by Shawn.
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2013-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2013-2015 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -24,10 +24,14 @@ import sumolib.net
 
 TAXI_STATUS_FREE_FLOW = "70"
 
+
 def fcd2gpsdat(inpFCD, outSTRM, further):
-  date = further["base-date"]
-  for timestep in inpFCD:
-    if timestep.vehicle:
-      mtime = str(date + datetime.timedelta(seconds=int(float(timestep.time)))) # does not work with subseconds
-      for v in timestep.vehicle:
-        print('\t'.join([v.id, mtime, str(v.x), str(v.y), TAXI_STATUS_FREE_FLOW,str(float(v.speed)*3.6)]), file=outSTRM)
+    date = further["base-date"]
+    for timestep in inpFCD:
+        if timestep.vehicle:
+            # does not work with subseconds
+            mtime = str(
+                date + datetime.timedelta(seconds=int(float(timestep.time))))
+            for v in timestep.vehicle:
+                print('\t'.join([v.id, mtime, str(v.x), str(
+                    v.y), TAXI_STATUS_FREE_FLOW, str(float(v.speed) * 3.6)]), file=outSTRM)
